//Externals functions, that links to low level library like opengl
#ifndef _EI_H_
#define _EI_H_

//Function provided by the interface
extern void EI_draw_triangle(float s1[3],float s2[3],float s3[3]);
extern void EI_init(int w, int h, int f);
extern void EI_close(void);
extern void EI_mainloop(void);
extern void EI_prerender(void);
extern void EI_postrender(void);
extern void EI_translate(float x, float y, float z);
extern void EI_rotate(float a_x,float a_y,float a_z);
extern void EI_testrender(void);
extern void EI_load_program_glsl(const char * vsfilename,const char * psfilename,unsigned int * shaderProgramId);
extern void EI_use_program_glsl(unsigned int shaderProgramId);
extern void EI_program_param_glsl(unsigned int programid,const char * varname,float * varvalue,int length);


//Render faces array using vertex buffer objects
extern void EI_render_triangle_buffer(float * vertexarray, float * texcoordsarray,int texturecount,float * colorarray,int size);
extern void EI_render_triangle_buffer_with_cubemap(float * vertexarray, float * texcoordsarray,float * texcubemap,int texturecount,int size);
extern void EI_render_triangle_buffer_with_shader(float * vertexarray, float * texcoordsarray, float * colorarray,int texturecount,
					int size,unsigned int vertexProgramId,unsigned int fragmentProgramId);
extern void EI_render_quad_buffer(float * vertexarray, float * texcoordsarray,int texturecount,float * colorarray,int size);
extern void EI_render_quad_buffer_with_cubemap(float * vertexarray, float * texcoordsarray,float * texcubemap,int texturecount,int size);
extern void EI_render_quad_buffer_with_shader(float * vertexarray, float * texcoordsarray,float * colorarray,int texturecount,
					int size,unsigned int vertexProgramId,unsigned int fragmentProgramId);
extern void EI_RegisterVertexBuffer(unsigned int * bufName,unsigned int byteSize,float * vertexbuffer);
extern void EI_UnregisterVertexBuffer(unsigned int bufName);
extern void EI_RegisterColorBuffer(unsigned int * bufName,unsigned int byteSize,float * colorbuffer);
extern void EI_UnregisterColorBuffer(unsigned int bufName);
extern void EI_RegisterTexCoordsBuffer(unsigned int * bufName,unsigned int byteSize,float * texcoordbuffer);
extern void EI_UnregisterTexCoordsBuffer(unsigned int bufName);
extern void EI_RegisterFaceBuffer(unsigned int * bufName,unsigned int byteSize,unsigned int * facebuffer);
extern void EI_UnregisterFaceBuffer(unsigned int bufName);


extern void EI_render_quad(unsigned int elementBufferName,unsigned int vertexBufferName,unsigned int texcoordsBufferName,
unsigned int colorBufferName,unsigned int size,unsigned int texturecount);
extern void EI_render_triangle(unsigned int elementBufferName,unsigned int vertexBufferName,unsigned int texcoordsBufferName,
unsigned int colorBufferName,unsigned int size,unsigned int texturecount);

//Function for generating a cube map
extern void EI_generate_normalisation_cubemap(int size,unsigned int * normalCubeMapId);

//Shader Functions
extern void EI_load_vertex_program(const char * filename,unsigned int * vertexProgramId);
extern void EI_load_fragment_program(const char * filename,unsigned int * fragmentProgramId);

//Function for shadow rendering
extern void EI_shadow_volume_step1(void);
extern void EI_shadow_volume_step2(void);
extern void EI_shadow_volume_step3(void);
extern void EI_shadow_volume_step4(void);
extern void EI_shadow_volume_step5(void);
extern void EI_frontquad(void);

//Function for per pixel lighting using dot3
extern void EI_setup_dot3(int normalMapId,unsigned int normalCubeMapId);
extern void EI_setup_dot3_texture(int textureId);
extern void EI_quit_dot3(void);
extern void EI_draw_local_space(float * vertex, float * tangent, float * binormal, float * normal);

extern int EI_getticks(void);

extern void EI_set_fog(float density, float r, float g, float b);
extern void EI_enable_fog(void);
extern void EI_disable_fog(void);

//Atomic opengl direct translation
extern void EI_begin_triangle(void);
extern void EI_begin_quad(void);
extern void EI_begin_quadstrip(void);
extern void EI_vertex(float x,float y,float z);
extern void EI_end(void);
extern void EI_setcolor(float r,float g,float b);
extern int EI_new_list(int i);
extern void EI_end_list(void);
extern void EI_call_list(int id);
extern void EI_push_matrix(void);
extern void EI_pop_matrix(void);
extern void EI_load_identitymatrix(void);
extern void EI_gen_textures(int n,unsigned int * textures_names);
extern void EI_bind_2D_texture(unsigned int texture_name);
extern void EI_tex_image2d24(unsigned char * pixels,int w,int h,int texture_name);
extern void EI_tex_image2d32(unsigned char * pixels,int w,int h,int texture_name);
extern void EI_texcoord_i(int x,int y);
extern void EI_texcoord_f(float fX,float fY);
extern void EI_normal(float x,float y,float z);
extern void EI_scale(float sx,float sy,float sz);
extern void EI_enable_texture(void);
extern void EI_delay(int iDuration);
extern void EI_enable_blending(void);
extern void EI_disable_blending(void);
extern void EI_set_active_texture0(void);
extern void EI_set_active_texture1(void);
extern void EI_set_inactive_texture1(void);
extern void EI_set_inactive_texture0(void);
extern void EI_texture0_texcoord_f(float fX,float fY);
extern void EI_enable_textureblending(void);
extern void EI_disable_textureblending(void);
extern void EI_set_active_texture(int i);
extern void EI_set_inactive_texture(int i);
extern void EI_enable_depthtest(void);
extern void EI_disable_depthtest(void);


extern char EI_mouse_down;
extern char EI_mouse_up;
extern char EI_mouse_left;
extern char EI_mouse_right;
extern char EI_keyb_down;
extern char EI_keyb_up;
extern char EI_keyb_left;
extern char EI_keyb_right;
extern char EI_keyb_space;
extern char EI_keyb_esc;


#endif
