//
// File: engine.h
// Created by: User <Email>
// Created on: Thu Nov 25 19:38:09 2004
//

#ifndef _ENGINE_H_
#define _ENGINE_H_

class objecttable;
class object;
class mobile;
class camera;
class force;
class forcetable;
class Texture2D;
class BmpFontManager;
class SpriteManager;
class SoundManager;
//class BufferManager;

#include "matrix33f.h"
#include "vector3f.h"
#include "light.h"
#include "face.h"

#include "BufferManager.h"
#include "TextureManager.h"
#include "ShaderManager.h"
class RenderManager;
class engine
{
	public:
		engine();
		 ~engine();

		// engine interface
			void setFog(float density,float r,float g, float b);
			void enableFog(bool value);
	  int add_object(object * obj);
      int add_object(mobile * m);
      int add_object(object * o, int id);
      int add_mobile(mobile * m);
      int add_mobile(mobile * m, int id);
      object * get_mobile(int id);
      object * get_object(int id);
	  int add_force(force * f);
	  int remove_object(int id);
	  int remove_mobile(int id);
	  int get_entitycount();
	  void set_camera(camera * cam);
	  camera * get_camera(void);
	  void init (int w, int h, int f);
	  void mainloop(void);
      void run(void);
	  void render(void);
	  void apply_camera(void);
	  void update_mobiles(float dt);
	  void apply_forces(void);
	  void set_processcallback(void (func)(void*), void*);
	  int add_texture(char * sFileName);
      int bind_texture(int iTextureIndex);
      Texture2D * get_texture_from_index(int iIndex);
	  Texture2D * get_texture_from_textureid(int iTextureID);
      int PrintText(char * sText, float fX, float fY, float fSize);
      float getFPS(void);
      int CreateSprite(int iTextureID, float fX, float fY, float fZ, float fWidth,float fHeight);
      int RemoveSprite(int iSpriteID);
      float * GetSpritePosition(int iSpriteID);
      float * GetSpriteOrientation(int iSpriteID);
      void SetTextureGlobalAlpha(int iTextureID,int value);

       //Audio stuff
      int LoadAudio(char * sFileName);
      int UnloadAudio(int iAudioID);
      int PlayAudio(char * sFileName);
      int PlayAudio(int iAudioID);
      int LoadMusic(char * sFileName);
      int PlayMusic(int iMusicID,int iLoops);

	void add_light(light * polight);
	//void calculate_lightings(void);
	void calculate_lightings(FaceGroup * fgrp);
	//int getFaceCount();
	int getVertexCount();
	int getMobileCount();
	int getObjectCount();
	int getQuadCount();
	int getTriangleCount();
	void translateCamera(float x,float y,float z);
	void rotateCamera(float ax,float ay, float az);
	void getObjectArray(object ** arrObj,int maxsize,int * length);
	protected:

      float fFPS;
     	int vertexcount;
		// engine variables
	  objecttable * objects;
	  objecttable * mobiles;
	  forcetable * forces;
      Texture2D ** vTexture2D;
	  camera * cam;
      BmpFontManager * poBmpFontManager;
      SpriteManager * poSpriteManager;
      SoundManager * poSoundManager;
      ShaderManager * shaderManager;
	  RenderManager * renderManager;

	  void (*callback_func)(void*);
      // callback_func;
	  void * callback_args;
	  int * display_list;
	  int * display_mobile_list;
 	  float ph_rate;


	//float arrayvertex[10000][3];

	//For quads
	//int quadfacecount;
	//int arrayquadface[1000][4];
	//int arrayquadtextureid[1000];
	//float arrayquadnormals[1000][3];
	//float arrayvertexcolor[10000][3];
	//void newrender(object * o);
	//void renderquads(void);
	//void renderfaces(void);

	//For triangles
	//int trianglefacecount;
	//int arraytriangleface[10000][3];
	//int arraytriangletextureid[10000];
	//float arraytrianglenormals[10000][3];

	//face arrayface[10000];
	//int facecount;
	//float arrayvertexcolor[1000][3];
	//void newrendertriangle(object * o);
	//void rendertriangles(void);


	//face arrayshadowface[10000];
	//int shadowfacecount;

	//Methods for shadow volumes
	//void buildshadowvolume(light * polight, int startface,int endface);
	void rendershadowfaces(void);
	//int startshadowface;
	//void addquad(float * a,float * b,float * c,float * d);
	light * arraylight[10];
	float ambientlight[3];

	unsigned int vertexprog;
	unsigned int fragmentprog;

	unsigned int normalCubeMapId;
	//float arraynormal[10000][3];
	//float arraybinormal[10000][3];
	//float arraytangent[10000][3];

	bool useShader;

	//New render structure
	//float bufferVertices[100000];
	//int offsetVertices;

	//unsigned int bufferFaceIndex[10000];
	//int offsetFaceIndex;

	//float bufferTexCoords[10000];
	//int offsetTexCoords;

	FaceGroup arrayFaceGroup[1000];
	int faceGroupCount;
	TextureManager * textureManager;
	BufferManager * bufferManager;
	BufferManager * staticBufferManager;
	//void AddFaceIndexItem(int size,int * faceindex);
	//void AddVertexItem(int size,float * vertex, DWORD * buffer);
	void AddTexCoords(int size,float * texcoords);
	void FaceGroupAdd(object * o);
	//void FaceGroupRender(FaceGroup * facegrp);
	void FaceGroupTransform(FaceGroup * facegrp);
	void ShadowFaceGroupInit(FaceGroup * facegrp);
	//void FaceGroupRegisterBuffers(FaceGroup * facegrp);
	//void FaceGroupUnregisterBuffers(FaceGroup * facegrp)	;
	//void FaceGroupAddQuad(FaceGroup * facegrp,float * a,float * b,float *c,float * d);

	void BuffersReset(void);
	void FaceGroupPrint(FaceGroup * facegrp);
	void renderFaceGroups();
	void ShadowTriangleFaceGroupInit(FaceGroup * facegrp);
	void buildshadowvolume(light * polight, FaceGroup * fgrp);


	//void FaceGroupAddTriangle(FaceGroup * facegrp,float * a,float * b,float *c);
};


#endif	//_ENGINE_H_
