//
// File: engine.cc
// Created by: User <Email>
// Created on: Thu Nov 25 19:38:09 2004
//

#include "engine.h"
#include "camera.h"
#include "objecttable.h"
#include "mobile.h"
#include "forcetable.h"
#include "force.h"
#include "gravity.h"
#include "ei.h"
#include <stdio.h>
#include "texture2d.h"
#include "BmpFontManager.h"
#include "SpriteManager.h"
#include "SoundManager.h"
#include <string.h>
#include "BufferManager.h"
#include "ShaderManager.h"
#include "RenderManager.h"

//TO BE REMOVED
//#include "terrain.h"

engine::engine()
{

	this->objects = new objecttable(10000);
	this->mobiles = new objecttable(10000);
	this->forces = new forcetable(5);
	//for (int i = 0; i i< this->forces->get_size();i++)

	//this->add_force(new gravity());
	this->display_list = new int[10000];
	this->display_mobile_list = new int[10000];
	this->cam = new camera();
	this->callback_func = NULL;
	ph_rate = 0.1;
    this->vTexture2D = new Texture2D*[200];
    for (int i = 0;i < 200;i++)
      this->vTexture2D[i] = NULL;
    this->poBmpFontManager = new BmpFontManager(this);

    this->poSpriteManager = new SpriteManager(this);

    this->poSoundManager = new SoundManager();
    this->poSoundManager->Init();
    	for (int i = 0;i < 10;i++)
	{
		this->arraylight[i] = NULL;

	}

	this->ambientlight[0] = 0.2;
	this->ambientlight[1] = 0.2;
	this->ambientlight[2] = 0.2;


  	this->vertexcount = 0;
	//this->quadfacecount=0;
	//this->trianglefacecount=0;

	this->useShader = true;


	 bufferManager = new BufferManager(2000000);
	 staticBufferManager = new BufferManager(1000000);
	 textureManager = new TextureManager(20);

	this->shaderManager = new ShaderManager(10);
	this->renderManager = new RenderManager(this->shaderManager);

}


engine::~engine()
{
	delete this->objects;
	delete this->cam;
	delete this->display_list;
    delete this->vTexture2D;
    delete this->poBmpFontManager;
    delete this->poSpriteManager;
}

int engine::add_object(object * obj){
  int id;
  id = objects->add_object(obj);
  if (obj->is_listable()){
      display_list[id] = EI_new_list(id+1);
      obj->set_idlist(display_list[id]);
      obj->draw();
      //printf("EXECUTED !\n");
      EI_end_list();
  }
  return id;
}

int engine::add_object(mobile * m){
  int id;
  id = mobiles->add_object(m);
  if (m->is_listable()){
      display_mobile_list[id] = EI_new_list(id+1);
      m->set_idlist(display_mobile_list[id]);
      m->draw();
      EI_end_list();
      return display_list[id];
  }

  return id;
}

int engine::add_object(object * o, int id){
  int idt = objects->add_object(o);
  if (o->is_listable()){
      display_list[idt] = id;
      o->set_idlist(display_list[idt]);
  }
  return idt;
}


int engine::add_mobile(mobile * m){
  int id;
  id = mobiles->add_object(m);
  if (m->is_listable()){
      //printf("vz %f\n",m->get_speed()[2]);
      display_mobile_list[id] = EI_new_list(id+1);
      m->set_idlist(display_mobile_list[id]);
      m->draw();
      EI_end_list();
  }
  return id;
}

int engine::add_mobile(mobile * m, int id){
  int idt = mobiles->add_object(m);
  if (m->is_listable()){
      display_mobile_list[idt] = id;
      m->set_idlist(display_mobile_list[idt]);
  }
  return idt;
}

int engine::get_entitycount(){
  return this->objects->get_count() + this->mobiles->get_count();
}


int engine::add_force(force * f){
  return forces->add_force(f);

}


int engine::remove_object(int id){
  return objects->remove_object(id);
}
int engine::remove_mobile(int id){
  return mobiles->remove_object(id);
}


void engine::set_camera(camera * cam){
  this->cam = cam;
}
camera * engine::get_camera(void){
  return this->cam;
}

void engine::init (int w, int h, int f){
  EI_init(w,h,f);


    int iTextureID = this->poBmpFontManager->LoadBmpFont((char *)"textures/062-32.bmp");
    //Texture2D * oTexture = this->get_texture_from_textureid(iTextureID);
    //oTexture->set_coloralphavalue(0,0,0,128);

    //this->poBmpFontManager->DisplayText("HELLO WORLD !",-3.75,2.885,0.1);

	//EI_generate_normalisation_cubemap(256,&this->normalCubeMapId);
    //EI_load_vertex_program("vertexprog.txt",&this->vertexprog);
    //EI_load_fragment_program("fragmentprog2.txt",&this->fragmentprog);
}
void engine::mainloop(void){
  while(1){
	EI_mainloop();
	if (this->callback_func != NULL)
	  this->callback_func(this->callback_args);
	this->render();
	this->apply_forces();
	this->update_mobiles(0.01);
  }
}

void engine::run(void){
  static int iCount = 0;
  static int iTicks = EI_getticks();
  iCount ++;

  EI_mainloop();

  if (this->callback_func != NULL)
	this->callback_func(this->callback_args);

  this->render();

  this->apply_forces();
  this->update_mobiles(0.01);

  this->poBmpFontManager->Clear();

  if (iCount == 20)
  {
    iTicks = EI_getticks() - iTicks;
     this->fFPS = 20000./(float)(iTicks);
     iTicks = EI_getticks();
     iCount = 0;

  }

  //CPU saving
 //EI_delay(5);
}

float engine::getFPS()
{
  return this->fFPS;
}

void engine::apply_camera(void){


  EI_rotate(this->get_camera()->get_orientation()[0],
               this->get_camera()->get_orientation()[1],
               this->get_camera()->get_orientation()[2]);



  EI_translate(-this->get_camera()->get_position()[0],
               -this->get_camera()->get_position()[1],
               -this->get_camera()->get_position()[2]);




}

void engine::apply_forces(void){
  for (int i = 0; i < this->mobiles->get_size();i++){
    if (this->mobiles->get_object(i) != 0){
      float new_accel[3] = {0.,0.,0.};
      float * accel;
      for (int j = 0;j < this->forces->get_size();j++){
        if (this->forces->get_force(j) != 0){
          accel = this->forces->get_force(j)->apply((mobile *)this->mobiles->get_object(i),this->ph_rate);
          new_accel[0] = new_accel[0] +  accel[0];
          new_accel[1] = new_accel[1] +  accel[1];
          new_accel[2] = new_accel[2] +  accel[2];
          delete accel;
        }
   }
        float * accel_mobile = ((mobile *)this->mobiles->get_object(i))->get_acceleration();
        accel_mobile[0] += new_accel[0];
        accel_mobile[1] += new_accel[1];
        accel_mobile[2] += new_accel[2];
        //printf("mobile acceleration: %f %f %f\n",accel_mobile[0],accel_mobile[1],accel_mobile[2]);


    }
  }
}


void engine::update_mobiles(float dt){
  mobile * m;

  for (int i = 0; i < this->mobiles->get_size();i++){
    if ((m = (mobile*)this->mobiles->get_object(i)) != 0){
      float * speed = m->get_speed();
      float * accel = m->get_acceleration();
      speed[0] += accel[0] * dt;
      speed[1] += accel[1] * dt;
      speed[2] += accel[2] * dt;
      accel[0] = 0.;
      accel[1] = 0.;
      accel[2] = 0.;
      float * position (m->get_position());
      position[0] = speed[0]* dt + position[0];
      position[1] = speed[1]* dt + position[1];
      position[2] = speed[2]* dt + position[2];
      float * orientation = m->get_orientation();


      float * angspeed = m->get_angularspeed();
      orientation[0] += (angspeed[0] * dt);
      orientation[1] += (angspeed[1] * dt);
      orientation[2] += (angspeed[2] * dt);

      object * o;
      int count = 0;
      for (int j = 0;(j < this->objects->get_size())&&(count < this->objects->get_count());j++)

          if ((o = this->objects->get_object(j)) != 0)
          {
            count++;

            if (o->collision_resolve(m->get_position()))
            {
               speed[0] = - speed[0];
               speed[1] = - speed[1];
               speed[2] = - speed[2];

            }


          }


    }
  }

}

object * engine::get_mobile(int id){
    return this->mobiles->get_object(id);
}


object * engine::get_object(int id){
    return this->objects->get_object(id);
}
void engine::set_processcallback(void (func)(void*), void* args){
  this->callback_func = func;
  this->callback_args = args;
}

void engine::calculate_lightings(FaceGroup * fgrp)
{
	fgrp->pColorBuffer = this->bufferManager->GetBuffer(fgrp->verticesCount*3);

	for (int i = 0;i < fgrp->verticesCount;i++)
	{
		fgrp->pColorBuffer[i*3].f = this->ambientlight[0];
		fgrp->pColorBuffer[i*3+1].f = this->ambientlight[1];
		fgrp->pColorBuffer[i*3+2].f = this->ambientlight[2];


		for (int j = 0;j < 10;j++)
		{
			if (this->arraylight[j] != NULL)
			{
				if (true)
				{

					vector3f facecolor = vector3f((float*)(fgrp->pColorBuffer + i*3));
					vector3f color;
					color = this->arraylight[j]->compute_lighting(
						vector3f((float*)(fgrp->pVertBuffer + i*3)),
						vector3f((float*)(fgrp->pNormalBuffer + i*3)));
					vector3f::add(facecolor,facecolor,color);

					//facecolor = vector3f((float*)(fgrp->pNormalBuffer + i*3));

					fgrp->pColorBuffer[i*3].f = facecolor(0);
					fgrp->pColorBuffer[i*3+1].f = facecolor(1);
					fgrp->pColorBuffer[i*3+2].f = facecolor(2);
				//printf("HERE %f %f %f\n",facecolor(0),facecolor(1),facecolor(2));

				}
			}

		}

	}

}

void engine::rendershadowfaces(void)
{
	this->renderManager->FaceGroupRegisterBuffers(&this->arrayFaceGroup[999]);
	this->renderManager->FaceGroupRender(&this->arrayFaceGroup[999]);
	this->renderManager->FaceGroupUnregisterBuffers(&this->arrayFaceGroup[999]);

}




void engine::buildshadowvolume(light * polight, FaceGroup * fgrp)
{

	bool * tFaceLight = new bool[fgrp->faceCount];
	int count = 0;
	int countlight = 0;
	for (int i = 0;i < fgrp->faceCount;i++)
	{
		tFaceLight[i] = false;
	}

	//printf("face count %d\n",this->facecount);
	//On cherche les faces illuminess
	for(int i = 0;i <  fgrp->faceCount;i++)
	{
		int a = fgrp->pFaceIndexBuffer[i*3].i*3;
		int b = fgrp->pFaceIndexBuffer[i*3+1].i*3;
		int c = fgrp->pFaceIndexBuffer[i*3+2].i*3;
		//printf("%d %d %d %f %f\n",a,b,c,((float*)(fgrp->pNormalBuffer+a))[0],fgrp->pNormalBuffer[a].f);
		/*
		vector3f normal_a = vector3f((float*)(fgrp->pNormalBuffer+a));
		vector3f normal_b = vector3f((float*)(fgrp->pNormalBuffer+b));
		vector3f normal_c = vector3f((float*)(fgrp->pNormalBuffer+c));
		vector3f normal = (normal_a + normal_b + normal_c);
		normal.normalize();
		normal.print();
		*/
	vector3f res;
	vector3f vm1;
	vector3f vm2;
		vector3f v0 = vector3f((float*)(fgrp->pVertBuffer+a));
		vector3f v1 = vector3f((float*)(fgrp->pVertBuffer+b));
		vector3f v2 = vector3f((float*)(fgrp->pVertBuffer+c));
		vector3f::sub(vm1,v1,v0);
		vector3f::sub(vm2,v2,v0);
		vector3f::vect(res,vm1,vm2);
		res.normalize();
		//res.print();
		vector3f normal;
		normal = res;

		vector3f vectolight;

		vector3f vf = (v0 + v1 +v2) * (1./3);

		//vector3f::sub(vectolight,polight->get_position(),vector3f(this->arrayvertex[this->arrayface[i].vertexindex[0]]));

		//vector3f::sub(vectolight,polight->get_position(),vector3f((float*)(fgrp->pVertBuffer+fgrp->pFaceIndexBuffer[i*3].i*3)));
		vector3f::sub(vectolight,polight->get_position(),vf);

		float scal = normal * vectolight;
	    //printf("Face %f\n",scal);
		if (scal > 0.0)
		{
			tFaceLight[i] = true;
			countlight++;

		} else
		{
				//FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[this->arrayface[i].vertexindex[0]],this->arrayvertex[this->arrayface[i].vertexindex[1]],this->arrayvertex[this->arrayface[i].vertexindex[2]]);

				FaceGroupAddTriangle(&this->arrayFaceGroup[999],(float*)(fgrp->pVertBuffer + fgrp->pFaceIndexBuffer[i*3].i*3),(float*)(fgrp->pVertBuffer + fgrp->pFaceIndexBuffer[i*3+1].i*3),(float*)(fgrp->pVertBuffer + fgrp->pFaceIndexBuffer[i*3+2].i*3));

		}

	}
	for(int i = 0;i < fgrp->faceCount;i++)
	{
		if (tFaceLight[i] == true)
		{

			// Pour chaque face illuminee, on cherche pour chaque arrete
			// si elle juxte un face non-illumine, elle appartient donc  la silouhette.
			for (int l = 0;l < 3;l++)
			{

				int a = fgrp->pFaceIndexBuffer[i*3+l].i*3;
				int b = fgrp->pFaceIndexBuffer[i*3+((l+1)%3)].i*3;


				for (int j = 0;j < fgrp->faceCount;j++)
				{

					if (tFaceLight[j] == false)
					{
						//Pour chaque face non illuminee et pour chaque arrete
						//de ces faces.
						for(int z = 0; z < 3;z++)
						{
							int a2 = fgrp->pFaceIndexBuffer[j*3+z].i*3;
							int b2 = fgrp->pFaceIndexBuffer[j*3+((z+1)%3)].i*3;

							if (((a==a2)&&(b==b2))||((a==b2)&&(b==a2)))
							{

								vector3f v1 = polight->projection(vector3f((float*)(fgrp->pVertBuffer+a)),100000.);
								vector3f v2 = polight->projection(vector3f((float*)(fgrp->pVertBuffer+b)),100000.);
								count++;

								//FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[a],this->arrayvertex[b],v2.get());
								//FaceGroupAddTriangle(&this->arrayFaceGroup[999],v2.get(),v1.get(),this->arrayvertex[a]);

								FaceGroupAddTriangle(&this->arrayFaceGroup[999],(float*)(fgrp->pVertBuffer+a),(float*)(fgrp->pVertBuffer+b),v2.get());
								FaceGroupAddTriangle(&this->arrayFaceGroup[999],v2.get(),v1.get(),(float*)(fgrp->pVertBuffer+a));


							}

						}

					}

				}

			}
		}
	}
	//printf("Objet -> %d arrete face illuminees: %d\n",count,countlight);
	delete tFaceLight;

	//printf("Objet -> %d arrete\n",count);
}



void engine::render(void){
  object * o;
  mobile * m;

	this->bufferManager->Reset();
  	this->vertexcount = 0;
	this->faceGroupCount = 0;

	//this->BuffersReset();

	this->ShadowTriangleFaceGroupInit(&this->arrayFaceGroup[999]);

  EI_prerender();

 this->apply_camera();


//Render Objects//
  for (int i = 0;i < this->objects->get_size();i++)
    if ((o = this->objects->get_object(i)) != 0){
      EI_push_matrix();

    //this->newrender(o);
	  o->render();
      EI_pop_matrix();
  }


//Render Mobiles//
  for (int i = 0;i < this->mobiles->get_size();i++)
    if ((m = (mobile*)this->mobiles->get_object(i)) != 0){
      //EI_push_matrix();


    //this->newrender(m);

	//NEW RENDER WITH FACEGROUP
	this->FaceGroupAdd(m);


      //EI_pop_matrix();
  }

  //NEW RENDERER WITH FACEGROUPS
  this->renderFaceGroups();
  //this->renderfaces();



  EI_shadow_volume_step1();
  		EI_setcolor(0.,0.,1.);
  EI_shadow_volume_step2();
  this->rendershadowfaces();

  EI_shadow_volume_step3();
  this->rendershadowfaces();
  EI_shadow_volume_step4();
  EI_frontquad();
   EI_shadow_volume_step5();

  //this->poSpriteManager->Render();

  EI_push_matrix();
  EI_load_identitymatrix();
 this->poBmpFontManager->Render();
  EI_pop_matrix();
  EI_postrender();

  /*
  if (EI_mouse_right)
    this->get_camera()->rotate_z(1);
  else if (EI_mouse_left)
    this->get_camera()->rotate_z(-1);
  //else rotate_z = rotate_z;

  if (EI_mouse_up)
    this->get_camera()->rotate_x(-1);
  else if (EI_mouse_down)
    this->get_camera()->rotate_x(+1);
*/

    float * pos;
    pos = this->get_camera()->get_position();


}

int engine::add_texture(char * sFileName)
{

  for (int i = 0;i < 200;i++)
  {
    if (this->vTexture2D[i] != NULL)
    {
       if (!strcmp(this->vTexture2D[i]->sFileName,sFileName))
       {
          return i;
       }
    }
  }
  Texture2D * poTexture2D = new Texture2D(sFileName);
  for (int i = 0;i < 200;i++)
  {
    if (this->vTexture2D[i] == NULL)
    {
        this->vTexture2D[i] = poTexture2D;
        printf("%s: %d",sFileName,this->vTexture2D[i]->iTextureID);
        //return iIndex;
        return i;
    }
  }
  //this->vTexture2D[iIndex] = poTexture2D;
  //return iIndex;
  return -1;
}
int engine::bind_texture(int iTextureIndex)
{
  Texture2D * poTexture2D = this->vTexture2D[iTextureIndex];
  return poTexture2D->bindTexture();
}

Texture2D * engine::get_texture_from_index(int iIndex)
{
   return this->vTexture2D[iIndex];
}



Texture2D * engine::get_texture_from_textureid(int iTextureID)
{
  for (int i = 0;i < 200;i++)
  {
    if (this->vTexture2D[i] != NULL)
    {
      if (this->vTexture2D[i]->iTextureID == iTextureID)
      return this->vTexture2D[i];
     }
   }
  return NULL;
}

int engine::PrintText(char * sText, float fX, float fY, float fSize)
{
  return this->poBmpFontManager->DisplayText(sText,fX,fY,fSize);
}

int engine::CreateSprite(int iTextureID, float fX, float fY, float fZ, float fWidth,float fHeight)
{
   return this->poSpriteManager->CreateSprite(iTextureID,fX,fY,fZ,fWidth,fHeight);
}

int engine::RemoveSprite(int iSpriteID)
{
   return this->poSpriteManager->RemoveSprite(iSpriteID);
}
float * engine::GetSpritePosition(int iSpriteID)
{
   return this->poSpriteManager->GetSpritePosition(iSpriteID);
}

float * engine::GetSpriteOrientation(int iSpriteID)
{
   return this->poSpriteManager->GetSpriteOrientation(iSpriteID);
}

void engine::SetTextureGlobalAlpha(int iTextureID,int value)
{
  this->vTexture2D[iTextureID]->set_globalalphavalue(value);
}


//Audio stuff
int engine::LoadAudio(char * sFileName){
  return this->poSoundManager->LoadAudio(sFileName);
}

int engine::UnloadAudio(int iAudioID){
  return this->poSoundManager->UnloadAudio(iAudioID);
}

int engine::PlayAudio(char * sFileName){
  return this->poSoundManager->PlayAudio(sFileName);
}

int engine::PlayAudio(int iAudioID){
  return this->poSoundManager->PlayAudio(iAudioID);
}

int engine::LoadMusic(char * sFileName){
  return this->poSoundManager->LoadMusic(sFileName);
}

int engine::PlayMusic(int iMusicID,int iLoops){
  return this->poSoundManager->PlayMusic(iMusicID,iLoops);
}


void engine::add_light(light * polight)
{
	for (int i = 0;i < 10;i++)
	{
		if (this->arraylight[i] == NULL)
		{
			this->arraylight[i] = polight;
			return;
		}

	}
}

void engine::FaceGroupTransform(FaceGroup * facegrp)
{
		matrix33f rotation;
		rotation = matrix33f::get_rotationmatrix(facegrp->rotation[0],facegrp->rotation[1],facegrp->rotation[2]);
		vector3f translation = vector3f(facegrp->position);
		unsigned int offset = 0;
		for (int i = 0;i < facegrp->verticesCount;i++)
		{
			vector3f v;

			if (facegrp->pVertBuffer != NULL)
			{
				v = vector3f((float*)(facegrp->pVertBuffer + offset));
				vector3f v2 = rotation * v + translation;
				facegrp->pVertBuffer[offset].f = v2(0);
				facegrp->pVertBuffer[offset+1].f = v2(1);
				facegrp->pVertBuffer[offset+2].f = v2(2);
			}

			if (facegrp->pNormalBuffer != NULL)
			{
				v =  vector3f((float*)(facegrp->pNormalBuffer + i*3));
				//v = rotation * v + translation;
				v = rotation * v;
				facegrp->pNormalBuffer[offset].f = v(0);
				facegrp->pNormalBuffer[offset+1].f = v(1);
				facegrp->pNormalBuffer[offset+2].f = v(2);
			}

			offset += 3;
		}

}

void engine::FaceGroupAdd(object * o)
{
	int startVerticesIndex;
	int endVerticesIndex;
	float vert[3];
	unsigned int progid;


	DWORD * bufferVertices = this->bufferManager->GetBuffer(o->vertexcount*3);
	DWORD * bufferNormals = this->bufferManager->GetBuffer(o->vertexcount*3);
	DWORD * bufferUVMaps = NULL;
	if (o->texcoordscount > 0)
	{
		bufferUVMaps = 	this->bufferManager->GetBuffer(o->texcoordscount*2);
		memcpy(bufferUVMaps,o->texcoords,sizeof(float)*o->texcoordscount*2);
	}

	if (o->textureCount > 0)
	{
		for (int i = 0;i < 	o->textureCount;i++)
		{
			/*
			if (o->textureid[i] == 0)
			{
				int ref = this->textureManager->add_texture(o->textureNames[i]);
				printf("New texture found: %s id=%d\n",o->textureNames[i],ref);
				o->textureid[i] = ref;
			}
			*/
		}
	}


/*
	if ((o->hasShader)&&(o->shader_prog_id == 0))
	{
		progid = this->shaderManager->add_shader(o->shader_prog);
		printf("progid %d\n",progid);
		o->shader_prog_id = progid;
	}
*/

	int offsetVertices = 0;

	for (int i = 0;i< o->vertexcount;i++)
	{
		int size = 3;

		bufferVertices[offsetVertices].f = o->vertices[i][0];
		bufferVertices[offsetVertices+1].f = o->vertices[i][1];
		bufferVertices[offsetVertices+2].f = o->vertices[i][2];


		if (o->normals != NULL)
		{
			bufferNormals[offsetVertices].f = o->normals[i][0];
			bufferNormals[offsetVertices+1].f = o->normals[i][1];
			bufferNormals[offsetVertices+2].f = o->normals[i][2];
		}

		offsetVertices += size;

	}


	//endVerticesIndex = this->offsetVertices;

	for(int i = 0;i < o->facegroupcount;i++)
	{

		this->arrayFaceGroup[this->faceGroupCount] = o->arrayfacegroup[i];

		for (int z = 0;z < o->textureCount;z++)
		{
	//		this->arrayFaceGroup[this->faceGroupCount].textureids[z] = o->textureid[z];

		}
		this->arrayFaceGroup[this->faceGroupCount].texturesCount = o->textureCount;

		this->arrayFaceGroup[this->faceGroupCount].pNormalBuffer = bufferNormals;

		this->arrayFaceGroup[this->faceGroupCount].pVertBuffer = bufferVertices;

		this->arrayFaceGroup[this->faceGroupCount].position[0] = o->get_position()[0];
		this->arrayFaceGroup[this->faceGroupCount].position[1] = o->get_position()[1];
		this->arrayFaceGroup[this->faceGroupCount].position[2] = o->get_position()[2];

		this->arrayFaceGroup[this->faceGroupCount].rotation[0] = o->get_orientation()[0];
		this->arrayFaceGroup[this->faceGroupCount].rotation[1] = o->get_orientation()[1];
		this->arrayFaceGroup[this->faceGroupCount].rotation[2] = o->get_orientation()[2];

		//printf("%f %f %f\n",o->get_orientation()[0],o->get_orientation()[1],o->get_orientation()[2]);
		/*
		for (int z = 0;z < o->texcoordscount;z++)
		{
			printf("u: %f v: %f\n",bufferUVMaps[z*2].f,bufferUVMaps[z*2+1].f);
		}
		*/
		/*
		if (o->shader_prog_id > 0)
		{
			this->arrayFaceGroup[this->faceGroupCount].shaderProgram = o->shader_prog_id;
			this->arrayFaceGroup[this->faceGroupCount].useShader = true;
		}
		*/
		this->arrayFaceGroup[this->faceGroupCount].uVertBufferOffset = offsetVertices;

		//this->arrayFaceGroup[this->faceGroupCount].verticesCount = offsetVertices;
		this->arrayFaceGroup[this->faceGroupCount].pUVMapBuffer = bufferUVMaps;
		this->arrayFaceGroup[this->faceGroupCount].uUVMapCount = o->texcoordscount;

		/*
		if ((o->hasShader == false)&&(o->shader_prog_id == 0))
			this->arrayFaceGroup[this->faceGroupCount].useSoftwareTransform = true;
		else this->arrayFaceGroup[this->faceGroupCount].useSoftwareTransform = false;
*/

		this->arrayFaceGroup[this->faceGroupCount].uFaceIndexBufferOffset = 0;
		this->arrayFaceGroup[this->faceGroupCount].pFaceIndexBuffer = this->bufferManager->GetBuffer(o->arrayfacegroup[i].uFaceIndexBufferOffset*3);
		for (int j = 0;j < o->arrayfacegroup[i].uFaceIndexBufferOffset;j++)
		{
			FaceGroupAddFaceIndexItem(&this->arrayFaceGroup[this->faceGroupCount],3,o->arrayface[j].vertexindex);
		}

		if (o->hasShadow)
		{
			for (int b = 0;b < 10;b++)
			{
				if (this->arraylight[b] != 0)
					this->buildshadowvolume(this->arraylight[b],&this->arrayFaceGroup[this->faceGroupCount]);
			}
		}


		this->faceGroupCount++;
	}

}

void engine::renderFaceGroups()
{
	for(int i = 0;i < this->faceGroupCount;i++)
	{
		if (this->arrayFaceGroup[i].useSoftwareTransform == true)
		{
			this->FaceGroupTransform(&this->arrayFaceGroup[i]);
		}
		this->calculate_lightings(&this->arrayFaceGroup[i]);

		this->renderManager->FaceGroupRegisterBuffers(&this->arrayFaceGroup[i]);

		this->renderManager->FaceGroupRender(&this->arrayFaceGroup[i]);
		this->renderManager->FaceGroupUnregisterBuffers(&this->arrayFaceGroup[i]);
	}
}


DWORD s_buffer1[100000];
DWORD s_buffer2[100000];

void engine::ShadowTriangleFaceGroupInit(FaceGroup * facegrp)
{
	facegrp->hasTexture = false;
	facegrp->hasNormalMap = false;
	//facegrp->startFacesIndex = this->offsetFaceIndex;
	//facegrp->endFacesIndex = this->offsetFaceIndex;
	facegrp->faceCount = 0;
	facegrp->texturesCount = 0;
	facegrp->useShader = 0;
	//facegrp->startTexCoordsIndex =0;
	//facegrp->endTexCoordsIndex =0;
	facegrp->texCoordsCount = 0;
	facegrp->verticesPerFace = 3;
	//facegrp->startVerticesIndex = this->offsetVertices;
	//facegrp->endVerticesIndex = this->offsetVertices;
	facegrp->verticesCount = 0;
	facegrp->texCoordsLayerCount = 0;
	facegrp->elementBufferName = 0;
	facegrp->vertexBufferName = 0;
	facegrp->texcoordsBufferName = 0;
	facegrp->colorBufferName = 0;
	//facegrp->pFaceIndexBuffer = this->bufferManager->GetBuffer(10000);
	//facegrp->pVertBuffer = this->bufferManager->GetBuffer(10000);
	facegrp->pFaceIndexBuffer = s_buffer1;
	facegrp->pVertBuffer = s_buffer2;
	facegrp->pColorBuffer = NULL;
	facegrp->pUVMapBuffer = NULL;
}




void engine::FaceGroupPrint(FaceGroup * facegrp)
{
	printf("-------------FaceGroup-------------\n");
	printf("facegrp->hasTexture %d\n",facegrp->hasTexture);
	printf("facegrp->hasNormalMap %d\n",facegrp->hasNormalMap);
	//printf("facegrp->startFacesIndex %d\n",facegrp->startFacesIndex);
	//printf("facegrp->endFacesIndex %d\n",facegrp->endFacesIndex);
	printf("facegrp->faceCount %d\n",facegrp->faceCount);
	printf("facegrp->texturesCount %d\n",facegrp->texturesCount);
	printf("facegrp->useShader %d\n",facegrp->useShader);
	//printf("facegrp->startTexCoordsIndex %d\n",facegrp->startTexCoordsIndex);
	//printf("facegrp->endTexCoordsIndex %d\n",facegrp->endTexCoordsIndex);
	printf("facegrp->texCoordsCount %d\n",facegrp->texCoordsCount);
	printf("facegrp->verticesPerFace %d\n",facegrp->verticesPerFace);
	//printf("facegrp->startVerticesIndex %d\n",facegrp->startVerticesIndex);
	//printf("facegrp->endVerticesIndex %d\n",facegrp->endVerticesIndex);
	printf("facegrp->verticesCount %d\n",facegrp->verticesCount);
	printf("facegrp->texCoordsLayerCount %d\n",facegrp->texCoordsLayerCount);
	printf("facegrp->elementBufferName %d\n",facegrp->elementBufferName);
	printf("facegrp->vertexBufferName %d\n",facegrp->vertexBufferName);
	printf("facegrp->texcoordsBufferName %d\n",facegrp->texcoordsBufferName);
	printf("facegrp->colorBufferName %d\n",facegrp->colorBufferName);
}
/*
void engine::BuffersReset(void)
{
	this->offsetVertices = 0;
}
*/

int engine::getVertexCount()
{
	return this->vertexcount;
}
int engine::getMobileCount()
{
	return this->mobiles->get_count();
}

int engine::getObjectCount()
{
	return this->objects->get_count();
}

void engine::translateCamera(float x,float y,float z)
{
	this->cam->translate_x(x);
	this->cam->translate_y(y);
	this->cam->translate_z(z);
}

void engine::rotateCamera(float ax,float ay, float az)
{
	this->cam->rotate_x(ax);
	this->cam->rotate_y(ay);
	this->cam->rotate_z(az);
}

void engine::getObjectArray(object ** arrObj,int maxsize,int * length)
{
	if (maxsize > this->mobiles->get_count())
	{
		*length = this->mobiles->get_count();
	} else *length = maxsize;

	//printf("Count: %d\n",*length);
	for (int i = 0;i <*length;i++)
	{
		arrObj[i] = this->mobiles->get_object(i);
	}
}


void engine::setFog(float density,float r,float g, float b)
{
	this->renderManager->setFog(density,r,g,b);
}

void engine::enableFog(bool value)
{
	this->renderManager->enableFog(value);
}
