//
// File: engine.cc
// Created by: User <Email>
// Created on: Thu Nov 25 19:38:09 2004
//

#include "engine.h"
#include "camera.h"
#include "objecttable.h"
#include "mobile.h"
#include "forcetable.h"
#include "force.h"
#include "gravity.h"
#include "ei.h"
#include <stdio.h>
#include "texture2d.h"
#include "BmpFontManager.h"
#include "SpriteManager.h"
#include "SoundManager.h"
#include <string.h>
#include "BufferManager.h"
#include "ShaderManager.h"


//TO BE REMOVED
//#include "terrain.h"

engine::engine()
{

	this->objects = new objecttable(10000);
	this->mobiles = new objecttable(10000);
	this->forces = new forcetable(5);
	//for (int i = 0; i i< this->forces->get_size();i++)
	  
	//this->add_force(new gravity());
	this->display_list = new int[10000];
	this->display_mobile_list = new int[10000];
	this->cam = new camera();
	this->callback_func = NULL;
	ph_rate = 0.1;
    this->vTexture2D = new Texture2D*[200];
    for (int i = 0;i < 200;i++)
      this->vTexture2D[i] = NULL;
    this->poBmpFontManager = new BmpFontManager(this);

    this->poSpriteManager = new SpriteManager(this);

    this->poSoundManager = new SoundManager();
    this->poSoundManager->Init();
    	for (int i = 0;i < 10;i++)
	{
		this->arraylight[i] = NULL;
		
	}
	
	this->ambientlight[0] = 0.8;
	this->ambientlight[1] = 0.8;
	this->ambientlight[2] = 0.8;	
	
	
  	this->vertexcount = 0;
	//this->quadfacecount=0;	
	//this->trianglefacecount=0;	
	
	this->useShader = true;
	    	
	    	
	 bufferManager = new BufferManager(100000);
	 staticBufferManager = new BufferManager(100000);
	 textureManager = new TextureManager(10);
	 
	this->shaderManager = new ShaderManager(10);
}


engine::~engine()
{
	delete this->objects;
	delete this->cam;
	delete this->display_list;
    delete this->vTexture2D;
    delete this->poBmpFontManager;
    delete this->poSpriteManager;
}

int engine::add_object(object * obj){
  int id;
  id = objects->add_object(obj);
  if (obj->is_listable()){
      display_list[id] = EI_new_list(id+1);
      obj->set_idlist(display_list[id]);
      obj->draw();
      //printf("EXECUTED !\n");
      EI_end_list();
  }
  return id;    
}

int engine::add_object(mobile * m){
  int id;
  id = mobiles->add_object(m);
  if (m->is_listable()){
      display_mobile_list[id] = EI_new_list(id+1);
      m->set_idlist(display_mobile_list[id]);
      m->draw();
      EI_end_list();
      return display_list[id];
  }
  
  return id;    
}    

int engine::add_object(object * o, int id){
  int idt = objects->add_object(o);
  if (o->is_listable()){
      display_list[idt] = id;
      o->set_idlist(display_list[idt]);
  }
  return idt;    
}  


int engine::add_mobile(mobile * m){
  int id;
  id = mobiles->add_object(m);
  if (m->is_listable()){
      //printf("vz %f\n",m->get_speed()[2]);
      display_mobile_list[id] = EI_new_list(id+1);
      m->set_idlist(display_mobile_list[id]);
      m->draw();
      EI_end_list();
  }
  return id;    
}

int engine::add_mobile(mobile * m, int id){
  int idt = mobiles->add_object(m);
  if (m->is_listable()){
      display_mobile_list[idt] = id;
      m->set_idlist(display_mobile_list[idt]);
  }
  return idt;    
}  

int engine::get_entitycount(){
  return this->objects->get_count() + this->mobiles->get_count();
}


int engine::add_force(force * f){
  return forces->add_force(f);
   
}


int engine::remove_object(int id){
  return objects->remove_object(id);
}
int engine::remove_mobile(int id){
  return mobiles->remove_object(id);
}


void engine::set_camera(camera * cam){
  this->cam = cam;
}
camera * engine::get_camera(void){
  return this->cam;
}

void engine::init (int w, int h, int f){
  EI_init(w,h,f);

    
    int iTextureID = this->poBmpFontManager->LoadBmpFont((char *)"textures/062-32.bmp");
    //Texture2D * oTexture = this->get_texture_from_textureid(iTextureID);
    //oTexture->set_coloralphavalue(0,0,0,128);

    //this->poBmpFontManager->DisplayText("HELLO WORLD !",-3.75,2.885,0.1);
	
	EI_generate_normalisation_cubemap(256,&this->normalCubeMapId);	
	
    EI_load_vertex_program("vertexprog.txt",&this->vertexprog);	
    EI_load_fragment_program("fragmentprog2.txt",&this->fragmentprog);
}
void engine::mainloop(void){
  while(1){
	EI_mainloop();	
	if (this->callback_func != NULL)
	  this->callback_func(this->callback_args);
	this->render();
	this->apply_forces();
	this->update_mobiles(0.01);
  }
}

void engine::run(void){
  static int iCount = 0;
  static int iTicks = EI_getticks();
  iCount ++;

  EI_mainloop();
	
  if (this->callback_func != NULL)
	this->callback_func(this->callback_args);
  
  this->render();
  
  this->apply_forces();
  this->update_mobiles(0.01);

  this->poBmpFontManager->Clear();

  if (iCount == 20)
  {
    iTicks = EI_getticks() - iTicks;
     this->fFPS = 20000./(float)(iTicks);
     iTicks = EI_getticks();
     iCount = 0;
     
  }

  //CPU saving
 //EI_delay(5);
}

float engine::getFPS()
{
  return this->fFPS;
}

void engine::apply_camera(void){


  EI_rotate(this->get_camera()->get_orientation()[0],
               this->get_camera()->get_orientation()[1],
               this->get_camera()->get_orientation()[2]);	



  EI_translate(-this->get_camera()->get_position()[0],
               -this->get_camera()->get_position()[1],
               -this->get_camera()->get_position()[2]);
	



}

void engine::apply_forces(void){
  for (int i = 0; i < this->mobiles->get_size();i++){
    if (this->mobiles->get_object(i) != 0){
      float new_accel[3] = {0.,0.,0.};
      float * accel;
      for (int j = 0;j < this->forces->get_size();j++){
        if (this->forces->get_force(j) != 0){
          accel = this->forces->get_force(j)->apply((mobile *)this->mobiles->get_object(i),this->ph_rate);
          new_accel[0] = new_accel[0] +  accel[0];
          new_accel[1] = new_accel[1] +  accel[1];
          new_accel[2] = new_accel[2] +  accel[2];
          delete accel;
        }
   }        
        float * accel_mobile = ((mobile *)this->mobiles->get_object(i))->get_acceleration();
        accel_mobile[0] += new_accel[0];
        accel_mobile[1] += new_accel[1];
        accel_mobile[2] += new_accel[2];
        //printf("mobile acceleration: %f %f %f\n",accel_mobile[0],accel_mobile[1],accel_mobile[2]);
                
          
    }    
  }    
}    


void engine::update_mobiles(float dt){
  mobile * m;

  for (int i = 0; i < this->mobiles->get_size();i++){
    if ((m = (mobile*)this->mobiles->get_object(i)) != 0){
      float * speed = m->get_speed();
      float * accel = m->get_acceleration();
      speed[0] += accel[0] * dt;
      speed[1] += accel[1] * dt;
      speed[2] += accel[2] * dt;
      accel[0] = 0.;
      accel[1] = 0.;
      accel[2] = 0.;
      float * position (m->get_position());
      position[0] = speed[0]* dt + position[0];
      position[1] = speed[1]* dt + position[1];
      position[2] = speed[2]* dt + position[2];
      float * orientation = m->get_orientation();
      
      
      float * angspeed = m->get_angularspeed();
      orientation[0] += (angspeed[0] * dt);
      orientation[1] += (angspeed[1] * dt);
      orientation[2] += (angspeed[2] * dt);

      object * o;
      int count = 0;
      for (int j = 0;(j < this->objects->get_size())&&(count < this->objects->get_count());j++)
          
          if ((o = this->objects->get_object(j)) != 0)
          {
            count++;

            if (o->collision_resolve(m->get_position()))
            {
               speed[0] = - speed[0];
               speed[1] = - speed[1];
               speed[2] = - speed[2];                                                                      
                                                                                   
            }
            
              
          }
      
      
    }    
  }    

}        

object * engine::get_mobile(int id){
    return this->mobiles->get_object(id);
}


object * engine::get_object(int id){
    return this->objects->get_object(id);
}
void engine::set_processcallback(void (func)(void*), void* args){
  this->callback_func = func;
  this->callback_args = args;
}

void engine::calculate_lightings(FaceGroup * fgrp)
{
	fgrp->pColorBuffer = this->bufferManager->GetBuffer(fgrp->verticesCount*3);
	
	for (int i = 0;i < fgrp->verticesCount;i++)
	{
		fgrp->pColorBuffer[i*3].f = this->ambientlight[0];
		fgrp->pColorBuffer[i*3+1].f = this->ambientlight[1];
		fgrp->pColorBuffer[i*3+2].f = this->ambientlight[2];				
	
	
		for (int j = 0;j < 10;j++)
		{
			if (this->arraylight[j] != NULL)
			{
				if (true)
				{
					
					vector3f facecolor = vector3f((float*)(fgrp->pColorBuffer + i*3));
					vector3f color;
					color = this->arraylight[j]->compute_lighting(
						vector3f((float*)(fgrp->pVertBuffer + i*3)),
						vector3f((float*)(fgrp->pNormalBuffer + i*3)));	
					vector3f::add(facecolor,facecolor,color);
					fgrp->pColorBuffer[i*3].f = facecolor(0);
					fgrp->pColorBuffer[i*3+1].f = facecolor(1);
					fgrp->pColorBuffer[i*3+2].f = facecolor(2);
																					
				}
			}
		}	
			
	}
	
}


void engine::calculate_lightings(void)
{
	for (int i = 0;i < this->facecount;i++)
	{	
		for (int j = 0; j < 4;j++)
		{
		this->arrayface[i].color[j][0] = this->ambientlight[0];
		this->arrayface[i].color[j][1] = this->ambientlight[1];
		this->arrayface[i].color[j][2] = this->ambientlight[2];			
		}
	}
	
	for (int i = 0;i < 10;i++)
	{
		if (this->arraylight[i] != NULL)
		{
			for (int k = 0;k < this->facecount;k++)
			{
				
				if (this->arrayface[k].useNormalMap == false)				
				{
					for (int l = 0;l < this->arrayface[k].nbvertices;l++)
					{
					vector3f facecolor = vector3f(this->arrayface[k].color[l]); 
					vector3f color;
						/*
					color = this->arraylight[i]->compute_lighting(
						vector3f(this->arrayvertex[this->arrayface[k].vertexindex[l]]),
						vector3f(this->arrayface[k].normal));
							*/
						//printf("LIGHT\n");
					
					color = this->arraylight[i]->compute_lighting(
						vector3f(this->arrayvertex[this->arrayface[k].vertexindex[l]]),
						vector3f(this->arraynormal[this->arrayface[k].vertexindex[l]]));						
						
					vector3f::add(facecolor,facecolor,color);
					this->arrayface[k].color[l][0] = facecolor(0);
					this->arrayface[k].color[l][1] = facecolor(1);
					this->arrayface[k].color[l][2] = facecolor(2);	
					}
				} 
				else
				{
					
					for (int l = 0;l < this->arrayface[k].nbvertices;l++)
					{					
						vector3f color;
						color = this->arraylight[i]->compute_pervertexlighting(
							vector3f(this->arrayvertex[this->arrayface[k].vertexindex[l]]),
							vector3f(this->arraynormal[this->arrayface[k].vertexindex[l]]),
							vector3f(this->arraytangent[this->arrayface[k].vertexindex[l]]),
							vector3f(this->arraybinormal[this->arrayface[k].vertexindex[l]]));	
						//puts("color");
						//color.print();
						//color = color * -1.;
								
						color.copy_to(this->arrayface[k].color[l]);

						
					}
					
				}
			}
			
/*			
			for (int k = 0;k < this->trianglefacecount;k++)
			{
				vector3f facecolor = vector3f(this->arrayvertexcolor[this->arraytriangleface[k][0]]); 
				vector3f color;
				color = this->arraylight[i]->compute_lighting(
					vector3f(this->arrayvertex[this->arraytriangleface[k][0]]),
					vector3f(this->arraytrianglenormals[k]));
							
				vector3f::add(facecolor,facecolor,color);
				this->arrayvertexcolor[this->arraytriangleface[k][0]][0] = facecolor(0);
				this->arrayvertexcolor[this->arraytriangleface[k][0]][1] = facecolor(1);
				this->arrayvertexcolor[this->arraytriangleface[k][0]][2] = facecolor(2);	

				facecolor = vector3f(this->arrayvertexcolor[this->arraytriangleface[k][1]]); 
				color =  this->arraylight[i]->compute_lighting(
					vector3f(this->arrayvertex[this->arraytriangleface[k][1]]),
					vector3f(this->arraytrianglenormals[k]));
				vector3f::add(facecolor,facecolor,color);
				this->arrayvertexcolor[this->arraytriangleface[k][1]][0] = facecolor(0);
				this->arrayvertexcolor[this->arraytriangleface[k][1]][1] = facecolor(1);
				this->arrayvertexcolor[this->arraytriangleface[k][1]][2] = facecolor(2);		

				facecolor = vector3f(this->arrayvertexcolor[this->arraytriangleface[k][2]]); 
				color =  this->arraylight[i]->compute_lighting(
					vector3f(this->arrayvertex[this->arraytriangleface[k][2]]),
					vector3f(this->arraytrianglenormals[k]));
				vector3f::add(facecolor,facecolor,color);
				this->arrayvertexcolor[this->arraytriangleface[k][2]][0] = facecolor(0);
				this->arrayvertexcolor[this->arraytriangleface[k][2]][1] = facecolor(1);
				this->arrayvertexcolor[this->arraytriangleface[k][2]][2] = facecolor(2);		
						
			}			
*/
		}
	}
}
/*
void engine::renderquads(void)
{
	this->calculate_lightings();
	float * color;	

	for (int i = 0;i <this->quadfacecount;i++)
	{
		EI_setcolor(1.,1.,1.);
		EI_bind_2D_texture(this->arrayquadtextureid[i]);
		EI_begin_quad();	
		
		color = this->arrayvertexcolor[this->arrayquadface[i][0]];
		EI_setcolor(color[0],color[1],color[2]);	
		
		EI_texcoord_i(0,0);
		EI_vertex(this->arrayvertex[this->arrayquadface[i][0]][0],this->arrayvertex[this->arrayquadface[i][0]][1],this->arrayvertex[this->arrayquadface[i][0]][2]);
		
		color = this->arrayvertexcolor[this->arrayquadface[i][1]];
		EI_setcolor(color[0],color[1],color[2]);			
		
		EI_texcoord_i(1,0);
		EI_vertex(this->arrayvertex[this->arrayquadface[i][1]][0],this->arrayvertex[this->arrayquadface[i][1]][1],this->arrayvertex[this->arrayquadface[i][1]][2]);
		
		color = this->arrayvertexcolor[this->arrayquadface[i][2]];
		EI_setcolor(color[0],color[1],color[2]);			
		
		EI_texcoord_i(1,1);
		EI_vertex(this->arrayvertex[this->arrayquadface[i][2]][0],this->arrayvertex[this->arrayquadface[i][2]][1],this->arrayvertex[this->arrayquadface[i][2]][2]);
		
		color = this->arrayvertexcolor[this->arrayquadface[i][3]];
		EI_setcolor(color[0],color[1],color[2]);		
		
		EI_texcoord_i(0,1);
		EI_vertex(this->arrayvertex[this->arrayquadface[i][3]][0],this->arrayvertex[this->arrayquadface[i][3]][1],this->arrayvertex[this->arrayquadface[i][3]][2]);
	
		EI_end();  
}
		
		EI_setcolor(1.,1.,1.);	
	}
	*/

void engine::renderfaces(void)
{
	for (int i = 0;i <this->facecount;i++)
	{
		if (this->arrayface[i].nbvertices == 4)
		{
/*
		for (int j = 0;j < 4;j ++)
		{	
			EI_draw_local_space(this->arrayvertex[this->arrayface[i].vertexindex[j]],
			this->arraytangent[this->arrayface[i].vertexindex[j]],
			this->arraybinormal[this->arrayface[i].vertexindex[j]],
			this->arraynormal[this->arrayface[i].vertexindex[j]]);		
		}
*/
		EI_setcolor(1.,1.,1.);
		
		if (this->arrayface[i].useNormalMap)
		{
		

				if (this->arrayface[i].nbtextures > 1)
				{
					//EI_set_inactive_texture0();			
					EI_set_active_texture0();
					EI_bind_2D_texture(this->arrayface[i].textureids[1]);
					//EI_set_inactive_texture1();				
				}
				else
				{
					EI_set_inactive_texture1();			
					EI_set_inactive_texture0();
				}			
			
				if (this->arrayface[i].nbtextures >= 2)
				{
	
					EI_set_active_texture1();		
					//EI_enable_textureblending();
					EI_bind_2D_texture(this->arrayface[i].textureids[2]);

				}	

			
				float vertex[12] ;
				float color[12];
				float texcoord[16];
				for (int j = 0;j < 4;j ++)
				{
					vertex[j*3] = this->arrayvertex[this->arrayface[i].vertexindex[j]][0];
					vertex[j*3+1] = this->arrayvertex[this->arrayface[i].vertexindex[j]][1];
					vertex[j*3+2] = this->arrayvertex[this->arrayface[i].vertexindex[j]][2];
					color[j*3] = 0.4;
					color[j*3+1] =0.4;
					color[j*3+2] = 0.4;
				}
				texcoord[0] = 0.;
				texcoord[1] = 0.;
				texcoord[2] = 1.;
				texcoord[3] = 0.;		
				texcoord[4] = 1.;
				texcoord[5] = 1.;
				texcoord[6] = 0.;
				texcoord[7] = 1.;			
		
				texcoord[8] = 0.;
				texcoord[9] = 0.;
				texcoord[10] = 1.;
				texcoord[11] = 0.;		
				texcoord[12] = 1.;
				texcoord[13] = 1.;
				texcoord[14] = 0.;
				texcoord[15] = 1.;	
				

		
				EI_render_quad_buffer( vertex,texcoord,this->arrayface[i].nbtextures - 1,color,1);
	

			}
		
		
		if (((this->useShader == false)&&(this->arrayface[i].useNormalMap==true))
			||(this->arrayface[i].useNormalMap==false))
		{
		//EI_enable_textureblending();
			if (this->arrayface[i].nbtextures > 0)
			{
				if (this->arrayface[i].useNormalMap)
				{
					EI_setup_dot3(this->arrayface[i].textureids[0],this->normalCubeMapId);
				
				} else 
				{
					//EI_set_inactive_texture0();			
					EI_set_active_texture0();
					EI_bind_2D_texture(this->arrayface[i].textureids[0]);
					//EI_set_inactive_texture1();				
				}
					
				if (this->arrayface[i].nbtextures >= 2)
				{
					if(this->arrayface[i].useNormalMap)
					{			
					
						EI_setup_dot3_texture(this->arrayface[i].textureids[1]);				
					} else 
					{
						EI_set_active_texture1();		
					//EI_enable_textureblending();
						EI_bind_2D_texture(this->arrayface[i].textureids[1]);
					}
				} else 
					{
					//EI_set_inactive_texture1();
				
					}
			}	
			else
			{
				puts("NO TEXTURE");
				EI_set_inactive_texture1();			
				EI_set_inactive_texture0();
			}
			
			float vertex[12] ;
			float color[12];
			float texcoord[16];
			for (int j = 0;j < 4;j ++)
			{
				vertex[j*3] = this->arrayvertex[this->arrayface[i].vertexindex[j]][0];
				vertex[j*3+1] = this->arrayvertex[this->arrayface[i].vertexindex[j]][1];
				vertex[j*3+2] = this->arrayvertex[this->arrayface[i].vertexindex[j]][2];
				color[j*3] = this->arrayface[i].color[j][0];
				color[j*3+1] = this->arrayface[i].color[j][1];
				color[j*3+2] = this->arrayface[i].color[j][2];
			}
			texcoord[0] = 0.;
			texcoord[1] = 0.;
			texcoord[2] = 1.;
			texcoord[3] = 0.;		
			texcoord[4] = 1.;
			texcoord[5] = 1.;
			texcoord[6] = 0.;
			texcoord[7] = 1.;			
			
			texcoord[8] = 0.;
			texcoord[9] = 0.;
			texcoord[10] = 1.;
			texcoord[11] = 0.;		
			texcoord[12] = 1.;
			texcoord[13] = 1.;
			texcoord[14] = 0.;
			texcoord[15] = 1.;	

			for (int u = 0;u < 16;u++)
			{
				//printf("kookoo");
				texcoord[u] = this->arrayface[i].texCoords[u];
			}
			

			if (this->arrayface[i].useNormalMap)
				EI_render_quad_buffer_with_cubemap( vertex,texcoord,color,this->arrayface[i].nbtextures,1);	
			else EI_render_quad_buffer( vertex,texcoord,this->arrayface[i].nbtextures,color,1);
			
			if (this->arrayface[i].nbtextures > 0)
			{
				
				if (this->arrayface[i].nbtextures >= 2)
				{
					//EI_disable_textureblending();
				
				}	
				
				if (this->arrayface[i].useNormalMap)
				{			
					EI_quit_dot3();
				}
				EI_set_inactive_texture0();
				EI_set_inactive_texture1();				
			}
		} 
		else
		{
			
			float vertex[12] ;
			float color[12];
			float texcoord[96];
			for (int j = 0;j < 4;j ++)
			{
				vertex[j*3] = this->arrayvertex[this->arrayface[i].vertexindex[j]][0];
				vertex[j*3+1] = this->arrayvertex[this->arrayface[i].vertexindex[j]][1];
				vertex[j*3+2] = this->arrayvertex[this->arrayface[i].vertexindex[j]][2];
				color[j*3] = this->arraylight[0]->color(0);
				color[j*3+1] = this->arraylight[0]->color(1);
				color[j*3+2] = this->arraylight[0]->color(2);
			}

			
			texcoord[0] = 0.;
			texcoord[1] = 0.;
			texcoord[2] = 0.;
			texcoord[3] = 0.;			
			texcoord[4] = 1.;
			texcoord[5] = 0.;
			texcoord[6] = 0.;
			texcoord[7] = 0.;			
			texcoord[8] = 1.;
			texcoord[9] = 1.;
			texcoord[10] = 0.;
			texcoord[11] = 0.;			
			texcoord[12] = 0.;
			texcoord[13] = 1.;			
			texcoord[14] = 0.;
			texcoord[15] = 0.;			

			for (int z = 0;z < 4;z++)
			{
				texcoord[z*4+16] = this->arraylight[0]->position(0);
				texcoord[z*4+17] = this->arraylight[0]->position(1);
				texcoord[z*4+18] = this->arraylight[0]->position(2);
				texcoord[z*4+19] = 1/2.;
			}

			
			for (int z = 0;z < 4;z++)
			{
				texcoord[z*4+32] =	this->arraytangent[this->arrayface[i].vertexindex[z]][0];
				texcoord[z*4+33] = this->arraytangent[this->arrayface[i].vertexindex[z]][1];
				texcoord[z*4+34] = this->arraytangent[this->arrayface[i].vertexindex[z]][2];
				texcoord[z*4+35] = 0.;
			}		

			for (int z = 0;z < 4;z++)
			{
				texcoord[z*4+48] =	this->arraybinormal[this->arrayface[i].vertexindex[z]][0];
				texcoord[z*4+49] = this->arraybinormal[this->arrayface[i].vertexindex[z]][1];
				texcoord[z*4+50] = this->arraybinormal[this->arrayface[i].vertexindex[z]][2];
				texcoord[z*4+51] = 0.;
			}	
	
			for (int z = 0;z < 4;z++)
			{
				texcoord[z*4+64] =	this->arraynormal[this->arrayface[i].vertexindex[z]][0];
				texcoord[z*4+65] = this->arraynormal[this->arrayface[i].vertexindex[z]][1];
				texcoord[z*4+66] = this->arraynormal[this->arrayface[i].vertexindex[z]][2];
				texcoord[z*4+67] = 0.;
			}		

			for (int z = 0;z < 4;z++)
			{
				texcoord[z*4+80] =	this->cam->get_position()[0];
				texcoord[z*4+81] =	this->cam->get_position()[1];
				texcoord[z*4+82] =	this->cam->get_position()[2];
				texcoord[z*4+83] = 0.;
			}				
			
			
			EI_set_active_texture0();
			EI_bind_2D_texture(this->arrayface[i].textureids[0]);
			
			EI_set_active_texture1();
			EI_bind_2D_texture(this->arrayface[i].textureids[1]);			
			
			EI_render_quad_buffer_with_shader( vertex,texcoord,color,6,1,this->vertexprog,this->fragmentprog);			

			EI_set_inactive_texture0();
			EI_set_inactive_texture1();
		}
		
}else if (this->arrayface[i].nbvertices == 3)
		{
		EI_setcolor(1.,1.,1.);
		
		if (this->arrayface[i].useNormalMap)
		{
		

				if (this->arrayface[i].nbtextures > 1)
				{
					//EI_set_inactive_texture0();			
					EI_set_active_texture0();
					EI_bind_2D_texture(this->arrayface[i].textureids[1]);
					//EI_set_inactive_texture1();				
				}
				else
				{
					EI_set_inactive_texture1();			
					EI_set_inactive_texture0();
				}			
			
				if (this->arrayface[i].nbtextures >= 2)
				{
	
					EI_set_active_texture1();		
					//EI_enable_textureblending();
					EI_bind_2D_texture(this->arrayface[i].textureids[2]);

				}	

			
				float vertex[9] ;
				float color[9];
				float texcoord[12];
				for (int j = 0;j < 3;j ++)
				{
					vertex[j*3] = this->arrayvertex[this->arrayface[i].vertexindex[j]][0];
					vertex[j*3+1] = this->arrayvertex[this->arrayface[i].vertexindex[j]][1];
					vertex[j*3+2] = this->arrayvertex[this->arrayface[i].vertexindex[j]][2];
					color[j*3] = 0.4;
					color[j*3+1] =0.4;
					color[j*3+2] = 0.4;
				}
				texcoord[0] = 0.;
				texcoord[1] = 0.;
				texcoord[2] = 1.;
				texcoord[3] = 0.;		
				texcoord[4] = 1.;
				texcoord[5] = 1.;
				texcoord[6] = 0.;
				texcoord[7] = 1.;			
		
				texcoord[8] = 0.;
				texcoord[9] = 0.;
				texcoord[10] = 1.;
				texcoord[11] = 0.;		
				texcoord[12] = 1.;
				texcoord[13] = 1.;
				texcoord[14] = 0.;
				texcoord[15] = 1.;	

		
				EI_render_quad_buffer( vertex,texcoord,this->arrayface[i].nbtextures - 1,color,1);
	

			}
		
		
		if (((this->useShader == false)&&(this->arrayface[i].useNormalMap==true))
			||(this->arrayface[i].useNormalMap==false))
		{
		//EI_enable_textureblending();
			if (this->arrayface[i].nbtextures > 0)
			{
				if (this->arrayface[i].useNormalMap)
				{
					EI_setup_dot3(this->arrayface[i].textureids[0],this->normalCubeMapId);
				
				} else 
				{
					//EI_set_inactive_texture0();			
					EI_set_active_texture0();
					EI_bind_2D_texture(this->arrayface[i].textureids[0]);
					//EI_set_inactive_texture1();				
				}
					
				if (this->arrayface[i].nbtextures >= 2)
				{
					if(this->arrayface[i].useNormalMap)
					{			
					
						EI_setup_dot3_texture(this->arrayface[i].textureids[1]);				
					} else 
					{
						EI_set_active_texture1();		
					//EI_enable_textureblending();
						EI_bind_2D_texture(this->arrayface[i].textureids[1]);
					}
				} else 
					{
					//EI_set_inactive_texture1();
				
					}
			}	
			else
			{
				//puts("NO TEXTURE");
				EI_set_inactive_texture1();			
				EI_set_inactive_texture0();
			}
			
			float vertex[9] ;
			float color[9];
			float texcoord[12];
			for (int j = 0;j < 3;j ++)
			{
				vertex[j*3] = this->arrayvertex[this->arrayface[i].vertexindex[j]][0];
				vertex[j*3+1] = this->arrayvertex[this->arrayface[i].vertexindex[j]][1];
				vertex[j*3+2] = this->arrayvertex[this->arrayface[i].vertexindex[j]][2];
				color[j*3] = this->arrayface[i].color[j][0];
				color[j*3+1] = this->arrayface[i].color[j][1];
				color[j*3+2] = this->arrayface[i].color[j][2];
				//printf("DRAW TRIANGLE FACE %d -> %f %f %f normal: %f %f %f color: %f %f %f\n",i,vertex[j*3],vertex[j*3+1],vertex[j*3+2]
				//,this->arrayface[i].normal[0],this->arrayface[i].normal[1],this->arrayface[i].normal[2],this->arrayface[i].color[j][0],this->arrayface[i].color[j][1],this->arrayface[i].color[j][2]);	
				/*
				color[j*3] = (float)j/3.;
				color[j*3+1] = (float)j/3.;
				color[j*3+2] = (float)j/3.;
				*/
				/*
				color[j*3] = 1.;
				color[j*3+1] = 1.;
				color[j*3+2] = 1.;		
				*/	
				//printf("DRAW TRIANGLE FACE %d -> %f %f %f\n",i,vertex[j*3],vertex[j*3+1],vertex[j*3+2]);	
			}
			texcoord[0] = 0.;
			texcoord[1] = 0.;
			texcoord[2] = 1.;
			texcoord[3] = 0.;		
			texcoord[4] = 1.;
			texcoord[5] = 1.;
			texcoord[6] = 0.;
			texcoord[7] = 1.;			
			
			texcoord[8] = 0.;
			texcoord[9] = 0.;
			texcoord[10] = 1.;
			texcoord[11] = 0.;		
			texcoord[12] = 1.;
			/*
			texcoord[13] = 1.;
			texcoord[14] = 0.;
			texcoord[15] = 1.;	
*/
			
			if (this->arrayface[i].useNormalMap)
				EI_render_triangle_buffer_with_cubemap( vertex,texcoord,color,this->arrayface[i].nbtextures,1);	
			else EI_render_triangle_buffer( vertex,texcoord,this->arrayface[i].nbtextures,color,1);
			
			if (this->arrayface[i].nbtextures > 0)
			{
				
				if (this->arrayface[i].nbtextures >= 2)
				{
					//EI_disable_textureblending();
				
				}	
				
				if (this->arrayface[i].useNormalMap)
				{			
					EI_quit_dot3();
				}
				EI_set_inactive_texture0();
				EI_set_inactive_texture1();				
			}
		} 
		else
		{
			
			float vertex[9] ;
			float color[9];
			float texcoord[72];
			for (int j = 0;j < 3;j ++)
			{
				vertex[j*3] = this->arrayvertex[this->arrayface[i].vertexindex[j]][0];
				vertex[j*3+1] = this->arrayvertex[this->arrayface[i].vertexindex[j]][1];
				vertex[j*3+2] = this->arrayvertex[this->arrayface[i].vertexindex[j]][2];
				color[j*3] = this->arraylight[0]->color(0);
				color[j*3+1] = this->arraylight[0]->color(1);
				color[j*3+2] = this->arraylight[0]->color(2);
			}

			
			texcoord[0] = 0.;
			texcoord[1] = 0.;
			texcoord[2] = 0.;
			texcoord[3] = 0.;			
			texcoord[4] = 1.;
			texcoord[5] = 0.;
			texcoord[6] = 0.;
			texcoord[7] = 0.;			
			texcoord[8] = 1.;
			texcoord[9] = 1.;
			texcoord[10] = 0.;
			texcoord[11] = 0.;			
			texcoord[12] = 0.;
			texcoord[13] = 1.;			
			texcoord[14] = 0.;
			texcoord[15] = 0.;			

			for (int z = 0;z < 3;z++)
			{
				texcoord[z*4+12] = this->arraylight[0]->position(0);
				texcoord[z*4+13] = this->arraylight[0]->position(1);
				texcoord[z*4+14] = this->arraylight[0]->position(2);
				texcoord[z*4+15] = 1/2.;
			}

			
			for (int z = 0;z < 3;z++)
			{
				texcoord[z*4+28] =	this->arraytangent[this->arrayface[i].vertexindex[z]][0];
				texcoord[z*4+29] = this->arraytangent[this->arrayface[i].vertexindex[z]][1];
				texcoord[z*4+30] = this->arraytangent[this->arrayface[i].vertexindex[z]][2];
				texcoord[z*4+31] = 0.;
			}		

			for (int z = 0;z < 3;z++)
			{
				texcoord[z*4+44] =	this->arraybinormal[this->arrayface[i].vertexindex[z]][0];
				texcoord[z*4+45] = this->arraybinormal[this->arrayface[i].vertexindex[z]][1];
				texcoord[z*4+46] = this->arraybinormal[this->arrayface[i].vertexindex[z]][2];
				texcoord[z*4+47] = 0.;
			}	
	
			for (int z = 0;z < 3;z++)
			{
				texcoord[z*4+60] =	this->arraynormal[this->arrayface[i].vertexindex[z]][0];
				texcoord[z*4+61] = this->arraynormal[this->arrayface[i].vertexindex[z]][1];
				texcoord[z*4+62] = this->arraynormal[this->arrayface[i].vertexindex[z]][2];
				texcoord[z*4+63] = 0.;
			}		

			for (int z = 0;z < 3;z++)
			{
				texcoord[z*4+80] =	this->cam->get_position()[0];
				texcoord[z*4+81] =	this->cam->get_position()[1];
				texcoord[z*4+82] =	this->cam->get_position()[2];
				texcoord[z*4+83] = 0.;
			}				
			
			printf("DRAW TRIANGLE FACE %d\n",i);
			EI_set_active_texture0();
			EI_bind_2D_texture(this->arrayface[i].textureids[0]);
			
			EI_set_active_texture1();
			EI_bind_2D_texture(this->arrayface[i].textureids[1]);			
			
			EI_render_triangle_buffer_with_shader( vertex,texcoord,color,6,1,this->vertexprog,this->fragmentprog);			

			EI_set_inactive_texture0();
			EI_set_inactive_texture1();
		}
		}	
	}
		
		EI_setcolor(1.,1.,1.);	
}


void engine::rendershadowfaces(void)
{
	//this->calculate_lightings();
	//float * color;	

	this->FaceGroupRegisterBuffers(&this->arrayFaceGroup[999]);
	this->FaceGroupRender(&this->arrayFaceGroup[999]);
	this->FaceGroupUnregisterBuffers(&this->arrayFaceGroup[999]);	
	
/*	
	for (int i = 0;i <this->shadowfacecount;i++)
	{


		if (this->arrayshadowface[i].nbvertices == 4)
		{		
		
		
		float vertex[12] ;
		float color[12];
		for (int j = 0;j < 4;j ++)
		{
			vertex[j*3] = this->arrayvertex[this->arrayshadowface[i].vertexindex[j]][0];
			vertex[j*3+1] = this->arrayvertex[this->arrayshadowface[i].vertexindex[j]][1];
			vertex[j*3+2] = this->arrayvertex[this->arrayshadowface[i].vertexindex[j]][2];
			color[j*3] = 1.;
			color[j*3+1] = 1.;
			color[j*3+2] = 1.;			
		}
		
		EI_render_quad_buffer( vertex,NULL,0,color,1);

		}else if (this->arrayshadowface[i].nbvertices == 3)
		{
		
		EI_setcolor(1.,1.,1.);
		EI_bind_2D_texture(this->arrayshadowface[i].textureids[0]);
		EI_begin_triangle();	
		
		color = this->arrayface[i].color[0];
		EI_setcolor(color[0],color[1],color[2]);	
		
		EI_texcoord_i(0,0);
		EI_vertex(this->arrayvertex[this->arrayshadowface[i].vertexindex[0]][0],this->arrayvertex[this->arrayshadowface[i].vertexindex[0]][1],this->arrayvertex[this->arrayshadowface[i].vertexindex[0]][2]);
		
		color = this->arrayface[i].color[1];
		EI_setcolor(color[0],color[1],color[2]);			
		
		EI_texcoord_i(1,0);
		EI_vertex(this->arrayvertex[this->arrayshadowface[i].vertexindex[1]][0],this->arrayvertex[this->arrayshadowface[i].vertexindex[1]][1],this->arrayvertex[this->arrayshadowface[i].vertexindex[1]][2]);
		
		color = this->arrayface[i].color[2];
		EI_setcolor(color[0],color[1],color[2]);			
		
		EI_texcoord_i(1,1);
		EI_vertex(this->arrayvertex[this->arrayshadowface[i].vertexindex[2]][0],this->arrayvertex[this->arrayshadowface[i].vertexindex[2]][1],this->arrayvertex[this->arrayshadowface[i].vertexindex[2]][2]);
		
		EI_end();	

		}	
	}
*/
		//EI_setcolor(1.,1.		,1.);	
}




/*
void engine::rendertriangles(void)
{
	this->calculate_lightings();
	float * color;	

	for (int i = 0;i <this->trianglefacecount;i++)
	{
		EI_setcolor(1.,1.,1.);
		EI_bind_2D_texture(this->arraytriangletextureid[i]);

			EI_begin_triangle();		
		
		//EI_normal(this->arraytrianglenormals[i][0],this->arraytrianglenormals[i][1],this->arraytrianglenormals[i][2]);
		color = this->arrayvertexcolor[this->arraytriangleface[i][0]];
		EI_setcolor(color[0],color[1],color[2]);	
		
		EI_texcoord_i(0,0);
		EI_vertex(this->arrayvertex[this->arraytriangleface[i][0]][0],this->arrayvertex[this->arraytriangleface[i][0]][1],this->arrayvertex[this->arraytriangleface[i][0]][2]);
		
		color = this->arrayvertexcolor[this->arraytriangleface[i][1]];
		EI_setcolor(color[0],color[1],color[2]);			
		
		EI_texcoord_i(1,0);
		EI_vertex(this->arrayvertex[this->arraytriangleface[i][1]][0],this->arrayvertex[this->arraytriangleface[i][1]][1],this->arrayvertex[this->arraytriangleface[i][1]][2]);
		
		color = this->arrayvertexcolor[this->arraytriangleface[i][2]];
		EI_setcolor(color[0],color[1],color[2]);			
		
		EI_texcoord_i(1,1);
		EI_vertex(this->arrayvertex[this->arraytriangleface[i][2]][0],this->arrayvertex[this->arraytriangleface[i][2]][1],this->arrayvertex[this->arraytriangleface[i][2]][2]);

		EI_end();  	
	}
		
		EI_setcolor(1.,1.,1.);	
}
*/

/*
void engine::addquad(float * a,float * b,float * c,float * d)
{

	this->arrayshadowface[this->shadowfacecount].nbtextures = 0;	
	
	this->arrayvertex[this->vertexcount][0] = a[0];
	this->arrayvertex[this->vertexcount][1] = a[1];
	this->arrayvertex[this->vertexcount][2] = a[2];	

	this->arrayshadowface[this->shadowfacecount].vertexindex[0] = this->vertexcount;
	this->vertexcount++;	
	
	this->arrayvertex[this->vertexcount][0] = b[0];
	this->arrayvertex[this->vertexcount][1] = b[1];
	this->arrayvertex[this->vertexcount][2] = b[2];	

	this->arrayshadowface[this->shadowfacecount].vertexindex[1] = this->vertexcount;
	this->vertexcount++;	

	this->arrayvertex[this->vertexcount][0] = c[0];
	this->arrayvertex[this->vertexcount][1] = c[1];
	this->arrayvertex[this->vertexcount][2] = c[2];	

	this->arrayshadowface[this->shadowfacecount].vertexindex[2] = this->vertexcount;
	this->vertexcount++;	

	this->arrayvertex[this->vertexcount][0] = d[0];
	this->arrayvertex[this->vertexcount][1] = d[1];
	this->arrayvertex[this->vertexcount][2] = d[2];	

	this->arrayshadowface[this->shadowfacecount].vertexindex[3] = this->vertexcount;
	
	this->arrayshadowface[this->shadowfacecount].nbvertices = 4;
	this->vertexcount++;	
	this->shadowfacecount++;
	//printf("shadow count: %d\n",this->shadowfacecount);

	
}*/

void engine::buildshadowvolume(light * polight, int startface,int endface)
{
	if (startface <  endface)
	{
	bool * tFaceLight = new bool[endface - startface];
	int count = 0;
	int countlight = 0;	
	for (int i = 0;i < (endface - startface);i++)
	{
		tFaceLight[i] = false;
	}
	
	//printf("face count %d\n",this->facecount);
	//On cherche les faces illuminess
	for(int i = startface;i <  endface;i++)
	{
		vector3f normal = vector3f(this->arrayface[i].normal);
		vector3f vectolight;
		
		//vector3f::sub(vectolight,vector3f(this->arrayvertex[this->arrayface[i].vertexindex[0]]),polight->get_position());
		vector3f::sub(vectolight,polight->get_position(),vector3f(this->arrayvertex[this->arrayface[i].vertexindex[0]]));
		float scal = normal * vectolight;
	    //printf("Face %f\n",scal);		
		if (scal > 0.0)
		{
			

			tFaceLight[i-startface] = true; 
			countlight++;
				//this->FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[this->arrayface[i].vertexindex[0]],this->arrayvertex[this->arrayface[i].vertexindex[1]],this->arrayvertex[this->arrayface[i].vertexindex[2]]);				
	
		} else 
		{
			//printf("kikoo\n");
/*
			this->addquad(this->arrayvertex[this->arrayface[i].vertexindex[3]],
				this->arrayvertex[this->arrayface[i].vertexindex[2]],
				this->arrayvertex[this->arrayface[i].vertexindex[1]],
				this->arrayvertex[this->arrayface[i].vertexindex[0]]);	
			

			this->addquad(this->arrayvertex[this->arrayface[i].vertexindex[0]],
				this->arrayvertex[this->arrayface[i].vertexindex[1]],
				this->arrayvertex[this->arrayface[i].vertexindex[2]],
				this->arrayvertex[this->arrayface[i].vertexindex[3]]);	
*/
			//printf("Add Quad %d",i);
			//Prevent shadow from darkening 
			/*
			this->FaceGroupAddQuad(&this->arrayFaceGroup[999],
				this->arrayvertex[this->arrayface[i].vertexindex[0]],
				this->arrayvertex[this->arrayface[i].vertexindex[1]],
				this->arrayvertex[this->arrayface[i].vertexindex[2]],
				this->arrayvertex[this->arrayface[i].vertexindex[3]]);
				*/
				
				FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[this->arrayface[i].vertexindex[0]],this->arrayvertex[this->arrayface[i].vertexindex[1]],this->arrayvertex[this->arrayface[i].vertexindex[2]]);		
					
		}
		
	}
	/*
	for(int i = startface;i < endface;i++)
	{
		if (tFaceLight[i-startface] == false)
		{		
			int countvoisin = 0;
			for (int l = 0;l < this->arrayface[i].nbvertices;l++)
			{
				
				int a = this->arrayface[i].vertexindex[l];
				int b = this->arrayface[i].vertexindex[(l+1)%this->arrayface[i].nbvertices];			
			

				for (int j = startface;j < endface;j++)
				{	
					
					if (tFaceLight[j-startface] == true)
					{
						//Pour chaque face non illuminee et pour chaque arrete
						//de ces faces.
						for(int z = 0; z < this->arrayface[j].nbvertices;z++)
						{
							int a2 = this->arrayface[j].vertexindex[z];
							int b2 = this->arrayface[j].vertexindex[(z+1)%this->arrayface[j].nbvertices];								
							
							if (((a==a2)&&(b==b2))||((a==b2)&&(b==a2)))
							{
								countvoisin++;
														
							}
								
						}
						
					}

				}
			
			}
			if (countvoisin != 3)
			{
				this->FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[this->arrayface[i].vertexindex[0]],this->arrayvertex[this->arrayface[i].vertexindex[1]],this->arrayvertex[this->arrayface[i].vertexindex[2]]);
			}
			
		}
	}
	*/
	for(int i = startface;i < endface;i++)
	{
		if (tFaceLight[i-startface] == true)
		{
			
			// Pour chaque face illuminee, on cherche pour chaque arrete 
			// si elle juxte un face non-illumine, elle appartient donc  la silouhette.			
			for (int l = 0;l < this->arrayface[i].nbvertices;l++)
			{
				
				int a = this->arrayface[i].vertexindex[l];
				int b = this->arrayface[i].vertexindex[(l+1)%this->arrayface[i].nbvertices];			
			

				for (int j = startface;j < endface;j++)
				{	
					
					if (tFaceLight[j-startface] == false)
					{
						//Pour chaque face non illuminee et pour chaque arrete
						//de ces faces.
						for(int z = 0; z < this->arrayface[j].nbvertices;z++)
						{
							int a2 = this->arrayface[j].vertexindex[z];
							int b2 = this->arrayface[j].vertexindex[(z+1)%this->arrayface[j].nbvertices];								
							
							if (((a==a2)&&(b==b2))||((a==b2)&&(b==a2)))
							{
								//printf("Arrete silouhette !");
								
								vector3f v1 = polight->projection(vector3f(this->arrayvertex[a]),100000.);
								vector3f v2 = polight->projection(vector3f(this->arrayvertex[b]),100000.);
								count++;
								vector3f normal1 = vector3f (this->arrayface[j].normal);
								
								vector3f normala = vector3f::normal(this->arrayvertex[a],this->arrayvertex[b],v2.get()); 
								//v1.print();
								/*
								if (normal1*normala < 0.)
									this->FaceGroupAddQuad(&this->arrayFaceGroup[999],this->arrayvertex[a],this->arrayvertex[b],v2.get(),v1.get());
								else
								*/
								
								//ORIGINAL
								//this->FaceGroupAddQuad(&this->arrayFaceGroup[999],this->arrayvertex[a],v1.get(),v2.get(),this->arrayvertex[b]);	
											
								//this->FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[a],v1.get(),v2.get());		
								
								FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[a],this->arrayvertex[b],v2.get());							
								FaceGroupAddTriangle(&this->arrayFaceGroup[999],v2.get(),v1.get(),this->arrayvertex[a]);										
								
								//this->FaceGroupAddTriangle(&this->arrayFaceGroup[999],v2.get(),this->arrayvertex[b],this->arrayvertex[a]);
								//this->FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[a],v1.get(),v2.get());	
								//this->FaceGroupAddQuad(&this->arrayFaceGroup[999],this->arrayvertex[a],this->arrayvertex[b],v2.get(),v1.get());
/*								
								if (normal1*normala < 0.)
									this->addquad(this->arrayvertex[a],this->arrayvertex[b],v2.get(),v1.get());
								else
								this->addquad(this->arrayvertex[a],v1.get(),v2.get(),this->arrayvertex[b]);
*/								
														
							}
								
						}
						
					}

				}
			
			}
		}
	}
	printf("Objet -> %d arrete face illuminees: %d\n",count,countlight);
	delete tFaceLight;
}
	//printf("Objet -> %d arrete\n",count);
}


void engine::buildshadowvolume(light * polight, FaceGroup * fgrp)
{

	bool * tFaceLight = new bool[fgrp->faceCount];
	int count = 0;
	int countlight = 0;	
	for (int i = 0;i < fgrp->faceCount;i++)
	{
		tFaceLight[i] = false;
	}
	
	//printf("face count %d\n",this->facecount);
	//On cherche les faces illuminess
	for(int i = 0;i <  fgrp->faceCount;i++)
	{
		int a = fgrp->pFaceIndexBuffer[i*3].i*3;
		int b = fgrp->pFaceIndexBuffer[i*3+1].i*3;
		int c = fgrp->pFaceIndexBuffer[i*3+2].i*3;
		//printf("%d %d %d %f %f\n",a,b,c,((float*)(fgrp->pNormalBuffer+a))[0],fgrp->pNormalBuffer[a].f);
		/*
		vector3f normal_a = vector3f((float*)(fgrp->pNormalBuffer+a));
		vector3f normal_b = vector3f((float*)(fgrp->pNormalBuffer+b));
		vector3f normal_c = vector3f((float*)(fgrp->pNormalBuffer+c));
		vector3f normal = (normal_a + normal_b + normal_c);
		normal.normalize();
		normal.print();
		*/
	vector3f res;
	vector3f vm1;
	vector3f vm2;		
		vector3f v0 = vector3f((float*)(fgrp->pVertBuffer+a));
		vector3f v1 = vector3f((float*)(fgrp->pVertBuffer+b));
		vector3f v2 = vector3f((float*)(fgrp->pVertBuffer+c));    		
		vector3f::sub(vm1,v1,v0);
		vector3f::sub(vm2,v2,v0);
		vector3f::vect(res,vm1,vm2);
		res.normalize();		
		//res.print();
		vector3f normal;
		normal = res;
		
		vector3f vectolight;
		
		vector3f vf = (v0 + v1 +v2) * (1./3);
		
		//vector3f::sub(vectolight,polight->get_position(),vector3f(this->arrayvertex[this->arrayface[i].vertexindex[0]]));
		
		//vector3f::sub(vectolight,polight->get_position(),vector3f((float*)(fgrp->pVertBuffer+fgrp->pFaceIndexBuffer[i*3].i*3)));
		vector3f::sub(vectolight,polight->get_position(),vf);
				
		float scal = normal * vectolight;
	    //printf("Face %f\n",scal);		
		if (scal > 0.0)
		{
			tFaceLight[i] = true; 
			countlight++;				
	
		} else 
		{
				//FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[this->arrayface[i].vertexindex[0]],this->arrayvertex[this->arrayface[i].vertexindex[1]],this->arrayvertex[this->arrayface[i].vertexindex[2]]);		
				
				FaceGroupAddTriangle(&this->arrayFaceGroup[999],(float*)(fgrp->pVertBuffer + fgrp->pFaceIndexBuffer[i*3].i*3),(float*)(fgrp->pVertBuffer + fgrp->pFaceIndexBuffer[i*3+1].i*3),(float*)(fgrp->pVertBuffer + fgrp->pFaceIndexBuffer[i*3+2].i*3));		
							
		}
		
	}
	for(int i = 0;i < fgrp->faceCount;i++)
	{
		if (tFaceLight[i] == true)
		{
			
			// Pour chaque face illuminee, on cherche pour chaque arrete 
			// si elle juxte un face non-illumine, elle appartient donc  la silouhette.			
			for (int l = 0;l < 3;l++)
			{
				
				int a = fgrp->pFaceIndexBuffer[i*3+l].i*3;
				int b = fgrp->pFaceIndexBuffer[i*3+((l+1)%3)].i*3;			
			

				for (int j = 0;j < fgrp->faceCount;j++)
				{	
					
					if (tFaceLight[j] == false)
					{
						//Pour chaque face non illuminee et pour chaque arrete
						//de ces faces.
						for(int z = 0; z < 3;z++)
						{
							int a2 = fgrp->pFaceIndexBuffer[j*3+z].i*3;
							int b2 = fgrp->pFaceIndexBuffer[j*3+((z+1)%3)].i*3;								
							
							if (((a==a2)&&(b==b2))||((a==b2)&&(b==a2)))
							{
								
								vector3f v1 = polight->projection(vector3f((float*)(fgrp->pVertBuffer+a)),100000.);
								vector3f v2 = polight->projection(vector3f((float*)(fgrp->pVertBuffer+b)),100000.);
								count++;
								
								//FaceGroupAddTriangle(&this->arrayFaceGroup[999],this->arrayvertex[a],this->arrayvertex[b],v2.get());							
								//FaceGroupAddTriangle(&this->arrayFaceGroup[999],v2.get(),v1.get(),this->arrayvertex[a]);								
								
								FaceGroupAddTriangle(&this->arrayFaceGroup[999],(float*)(fgrp->pVertBuffer+a),(float*)(fgrp->pVertBuffer+b),v2.get());							
								FaceGroupAddTriangle(&this->arrayFaceGroup[999],v2.get(),v1.get(),(float*)(fgrp->pVertBuffer+a));										
											
														
							}
								
						}
						
					}

				}
			
			}
		}
	}
	//printf("Objet -> %d arrete face illuminees: %d\n",count,countlight);
	delete tFaceLight;
	
	//printf("Objet -> %d arrete\n",count);
}


void engine::newrender(object * o)
{
	int startvertexcount = this->vertexcount;	
	int startfacecount = this->facecount;
	matrix33f rotation = matrix33f::get_rotationmatrix(o->get_orientation()[0],o->get_orientation()[1],o->get_orientation()[2]);	
	//printf("NEWRENDER 1 %d\n",o->vertexcount);
	for (int i = 0;i < o->vertexcount;i++)
	{
		vector3f translation = vector3f(o->get_position());
		vector3f vertex = vector3f(o->vertices[i]);
		
		vector3f result = (rotation * vertex)+translation;
		//vector3f::add(result,result,translation); 
		result.copy_to(this->arrayvertex[vertexcount]);
		/*
		this->arrayvertex[vertexcount][0] = result(0);
		this->arrayvertex[vertexcount][1] = result(1);
		this->arrayvertex[vertexcount][2] = result(2);	
		*/

		if (o->binormals != NULL)
		{
			//vector3f translation = vector3f(o->get_position());
			
			vector3f vertex = vector3f(o->binormals[i]);
			//vector3f result = rotation * vertex;
			
			vector3f result = (rotation * vertex);			
			//vector3f::add(result,result,translation); 			
			//result = vertex ;
			result.copy_to(this->arraybinormal[vertexcount]);		
/*			
			this->arraybinormal[vertexcount][0] = result(0);
			this->arraybinormal[vertexcount][1] = result(1);
			this->arraybinormal[vertexcount][2] = result(2);	*/	
		}
		
		
		if (o->normals != NULL)
		{
			//vector3f translation = vector3f(o->get_position());
			vector3f vertex = vector3f(o->normals[i]);
			//vector3f result = rotation * vertex;
			vector3f result = (rotation * vertex);			
			//vector3f::add(result,result,translation); 				
			//result = vertex ;
			//result.normalize();
			result.copy_to(this->arraynormal[vertexcount]);			
			/*
			this->arraynormal[vertexcount][0] = result(0);
			this->arraynormal[vertexcount][1] = result(1);
			this->arraynormal[vertexcount][2] = result(2);	*/	
		}

		if (o->tangents != NULL)
		{
			//vector3f translation = vector3f(o->get_position());
			vector3f vertex = vector3f(o->tangents[i]);
			//if (i == 0)
			//vertex.print();
			//vector3f result = rotation * vertex;
			vector3f result = (rotation * vertex);
			//vector3f::add(result,result,translation); 				
			//result = vertex ;
			
			//result.print();
			
			result.copy_to(this->arraytangent[vertexcount]);
			/*
			this->arraytangent[vertexcount][0] = result(0);
			this->arraytangent[vertexcount][1] = result(1);
			this->arraytangent[vertexcount][2] = result(2);	
*/			
		}
		
		this->vertexcount++;

	}
	//printf("NEWRENDER 2 %d\n",this->vertexcount);			
	for (int k = 0;k < o->facecount;k++)
	{
		if (o->arrayface[k].nbvertices == 3)
		{
		/*
		this->arraytriangleface[this->trianglefacecount][0] = o->arrayface[k].vertexindex[0] +startvertexcount;
		this->arraytriangleface[this->trianglefacecount][1] =  o->arrayface[k].vertexindex[1] + startvertexcount;
		this->arraytriangleface[this->trianglefacecount][2] =  o->arrayface[k].vertexindex[2] + startvertexcount;
		*/	
		this->arrayface[this->facecount].vertexindex[0] = o->arrayface[k].vertexindex[0] +startvertexcount;
		this->arrayface[this->facecount].vertexindex[1] = o->arrayface[k].vertexindex[1] +startvertexcount;
		this->arrayface[this->facecount].vertexindex[2] = o->arrayface[k].vertexindex[2] +startvertexcount;			
		
		//this->arraytriangletextureid[this->trianglefacecount] = o->arrayface[k].textureids[0];	
		for (int l = 0;l < o->arrayface[k].nbtextures;l++)
		this->arrayface[this->facecount].textureids[l] = o->arrayface[k].textureids[l];
		this->arrayface[this->facecount].nbtextures = o->arrayface[k].nbtextures;	
		
		vector3f vertex = vector3f(o->arrayface[k].normal);		
		vector3f result = rotation * vertex;		
		this->arrayface[this->facecount].normal[0] = result(0);
		this->arrayface[this->facecount].normal[1] = result(1);
		this->arrayface[this->facecount].normal[2] = result(2);	

		this->arrayface[this->facecount].nbvertices = 3;
		this->facecount++;			
		//this->trianglefacecount++;
		} else 
		if (o->arrayface[k].nbvertices == 4)
		{
			for (int u = 0;u < 16;u++)
				this->arrayface[this->facecount].texCoords[u] = o->arrayface[k].texCoords[u];

			this->arrayface[this->facecount].useNormalMap = o->hasNormalMap;
			this->arrayface[this->facecount].vertexindex[0] = o->arrayface[k].vertexindex[0] +startvertexcount;
			this->arrayface[this->facecount].vertexindex[1] = o->arrayface[k].vertexindex[1] +startvertexcount;
			this->arrayface[this->facecount].vertexindex[2] = o->arrayface[k].vertexindex[2] +startvertexcount;	
			this->arrayface[this->facecount].vertexindex[3] = o->arrayface[k].vertexindex[3] +startvertexcount;
			/*
			if (o->hasNormalMap)
			{
				for (int l = 1;l < o->arrayface[k].nbtextures;l++)
					this->arrayface[this->facecount].textureids[l-1] = o->arrayface[k].textureids[l];
				this->arrayface[this->facecount].nbtextures = o->arrayface[k].nbtextures - 1;	

				this->arrayface[this->facecount].normal[0] = 0.;
				this->arrayface[this->facecount].normal[1] = 0.;
				this->arrayface[this->facecount].normal[2] = 0.;					
			}	
			else
			*/
			{
				for (int l = 0;l < o->arrayface[k].nbtextures;l++)
					this->arrayface[this->facecount].textureids[l] = o->arrayface[k].textureids[l];
				this->arrayface[this->facecount].nbtextures = o->arrayface[k].nbtextures;
				vector3f vertex = vector3f(o->arrayface[k].normal);		
				vector3f result = rotation * vertex;		

			
				this->arrayface[this->facecount].normal[0] = result(0);
				this->arrayface[this->facecount].normal[1] = result(1);
				this->arrayface[this->facecount].normal[2] = result(2);	
				
			}
				//printf("texturecount: %d\n", o->arrayface[k].nbtextures);
		

			this->arrayface[this->facecount].nbvertices = 4;
			
		
			
			this->facecount++;			
		
		}
	}

	if (o->hasShadow)
		this->buildshadowvolume(this->arraylight[0],startfacecount,this->facecount);
	this->calculate_lightings();		
}


void engine::render(void){
  object * o;
  mobile * m;

	this->bufferManager->Reset();
  	this->vertexcount = 0;
	//this->quadfacecount=0;		
	//this->trianglefacecount=0;
	this->facecount = 0;
	//this->shadowfacecount = 0;
	this->faceGroupCount = 0;

	this->BuffersReset();

	this->ShadowTriangleFaceGroupInit(&this->arrayFaceGroup[999]);
	
  EI_prerender();
  
  
  //EI_testrender();
  // this->poBmpFontManager->Render();

 this->apply_camera();
   

//Render Objects//
  for (int i = 0;i < this->objects->get_size();i++)
    if ((o = this->objects->get_object(i)) != 0){
      EI_push_matrix();

    this->newrender(o);

      EI_pop_matrix();
  }


//Render Mobiles//
  for (int i = 0;i < this->mobiles->get_size();i++)
    if ((m = (mobile*)this->mobiles->get_object(i)) != 0){
      EI_push_matrix();

    
    //this->newrender(m);

	//NEW RENDER WITH FACEGROUP
	this->FaceGroupAdd(m);


      EI_pop_matrix();
  }      

  //NEW RENDERER WITH FACEGROUPS
  this->renderFaceGroups();
  this->renderfaces();



  EI_shadow_volume_step1();   
  		EI_setcolor(0.,0.,1.);
  EI_shadow_volume_step2(); 
  this->rendershadowfaces();
  
  EI_shadow_volume_step3(); 
  this->rendershadowfaces(); 
  EI_shadow_volume_step4(); 
  EI_frontquad();   
   EI_shadow_volume_step5();  
  
  //this->poSpriteManager->Render();

  EI_push_matrix();
  EI_load_identitymatrix();
 this->poBmpFontManager->Render();
  EI_pop_matrix();
  EI_postrender();
  
  /*
  if (EI_mouse_right)
    this->get_camera()->rotate_z(1);
  else if (EI_mouse_left)
    this->get_camera()->rotate_z(-1);
  //else rotate_z = rotate_z;
  
  if (EI_mouse_up)
    this->get_camera()->rotate_x(-1);
  else if (EI_mouse_down)
    this->get_camera()->rotate_x(+1);
*/
      
    float * pos;
    pos = this->get_camera()->get_position();    
    

}

int engine::add_texture(char * sFileName)
{

  for (int i = 0;i < 200;i++)
  {
    if (this->vTexture2D[i] != NULL)
    {
       if (!strcmp(this->vTexture2D[i]->sFileName,sFileName))
       {
          return i;
       }
    }
  }
  Texture2D * poTexture2D = new Texture2D(sFileName);
  for (int i = 0;i < 200;i++)
  {
    if (this->vTexture2D[i] == NULL)
    {
        this->vTexture2D[i] = poTexture2D;
        printf("%s: %d",sFileName,this->vTexture2D[i]->iTextureID);
        //return iIndex;
        return i;
    }
  }
  //this->vTexture2D[iIndex] = poTexture2D;
  //return iIndex;
  return -1;
}
int engine::bind_texture(int iTextureIndex)
{
  Texture2D * poTexture2D = this->vTexture2D[iTextureIndex];
  return poTexture2D->bindTexture(); 
}

Texture2D * engine::get_texture_from_index(int iIndex)
{
   return this->vTexture2D[iIndex];
}



Texture2D * engine::get_texture_from_textureid(int iTextureID)
{
  for (int i = 0;i < 200;i++)
  {
    if (this->vTexture2D[i] != NULL)
    { 
      if (this->vTexture2D[i]->iTextureID == iTextureID)
      return this->vTexture2D[i];
     }
   }
  return NULL;
}

int engine::PrintText(char * sText, float fX, float fY, float fSize)
{
  return this->poBmpFontManager->DisplayText(sText,fX,fY,fSize);
}

int engine::CreateSprite(int iTextureID, float fX, float fY, float fZ, float fWidth,float fHeight)
{
   return this->poSpriteManager->CreateSprite(iTextureID,fX,fY,fZ,fWidth,fHeight);
}

int engine::RemoveSprite(int iSpriteID)
{
   return this->poSpriteManager->RemoveSprite(iSpriteID);
}
float * engine::GetSpritePosition(int iSpriteID)
{
   return this->poSpriteManager->GetSpritePosition(iSpriteID);
}

float * engine::GetSpriteOrientation(int iSpriteID)
{
   return this->poSpriteManager->GetSpriteOrientation(iSpriteID);
}

void engine::SetTextureGlobalAlpha(int iTextureID,int value)
{
  this->vTexture2D[iTextureID]->set_globalalphavalue(value);
}


//Audio stuff
int engine::LoadAudio(char * sFileName){
  return this->poSoundManager->LoadAudio(sFileName);
}

int engine::UnloadAudio(int iAudioID){
  return this->poSoundManager->UnloadAudio(iAudioID);
}

int engine::PlayAudio(char * sFileName){
  return this->poSoundManager->PlayAudio(sFileName);
}

int engine::PlayAudio(int iAudioID){
  return this->poSoundManager->PlayAudio(iAudioID);
}

int engine::LoadMusic(char * sFileName){
  return this->poSoundManager->LoadMusic(sFileName);
}

int engine::PlayMusic(int iMusicID,int iLoops){
  return this->poSoundManager->PlayMusic(iMusicID,iLoops);
}


void engine::add_light(light * polight)
{
	for (int i = 0;i < 10;i++)
	{
		if (this->arraylight[i] == NULL)
		{
			this->arraylight[i] = polight;
			return;	
		}
		
	}
}

/*
void engine::AddFaceIndexItem(int size,int * faceindex)
{
	for (int i = 0;i < size;i++)
	{
		//this->bufferFaceIndex[this->offsetFaceIndex] = faceindex[i]/3;
		this->bufferFaceIndex[this->offsetFaceIndex] = faceindex[i];
		
		//printf("bufferFaceIndex %d %d -[ %f %f %f ]\n",this->offsetFaceIndex,faceindex[i],
		//this->bufferVertices[faceindex[i]],this->bufferVertices[faceindex[i]+1],this->bufferVertices[faceindex[i]+2]);		
		this->offsetFaceIndex++;
	}
}
*/

/*
void engine::AddVertexItem(int size,float * vertex,DWORD * buffer)
{
	//vector3f v =  vector3f(vertex);
	//v.print();
	

	for (int i = 0;i < size;i++)
	{
		this->bufferVertices[this->offsetVertices] =  vertex[i];
		//printf("%d %f\n",this->offsetVertices,this->bufferVertices[this->offsetVertices]);
		this->offsetVertices++;
	}
	
}
*/

/*
void engine::AddTexCoords(int size,float * texcoords)
{
	for (int i = 0;i < size;i++)
	{
		this->bufferTexCoords[this->offsetTexCoords] =  texcoords[i];
		this->offsetTexCoords++;
	}	
}
*/
/*
void engine::FaceGroupAdd(object * o)
{
	if (o->verticesPerFace == 4)
	{
		
		//Adding object vertices to the engine's buffer
		this->arrayFaceGroup[this->faceGroupCount].startVerticesIndex = this->offsetVertices;
		
		for (int i = 0;i< o->vertexcount;i++)
		{
			this->AddVertexItem(3,o->vertices[i]);
		}

		this->arrayFaceGroup[this->faceGroupCount].endVerticesIndex = this->offsetVertices;		
		
	
		//Adding object face index to the engine's face vertices index buffer		
		this->arrayFaceGroup[this->faceGroupCount].startFacesIndex = this->offsetFaceIndex;
		
		for (int i = 0;i< o->facecount;i++)
		{
			this->AddFaceIndexItem(o->verticesPerFace,o->arrayface[i].vertexindex);
		}

		this->arrayFaceGroup[this->faceGroupCount].endFacesIndex = this->offsetFaceIndex;			
		
		for (int i = this->arrayFaceGroup[this->faceGroupCount].startFacesIndex;i < this->arrayFaceGroup[this->faceGroupCount].endFacesIndex;i++)
		{
			this->bufferFaceIndex[i] += this->arrayFaceGroup[this->faceGroupCount].startVerticesIndex;			
		}
		
		//Adding object texcoords to the engine texcoords bufferFaceIndex
		this->arrayFaceGroup[this->faceGroupCount].startTexCoordsIndex = this->offsetTexCoords;
		
		
		for (int i = 0;i< o->texcoordscount;i++)
		{
			this->AddTexCoords(3,o->texcoords);
			//o->copyTexCoords(this->bufferTexCoords + this->offsetTexCoords);
		}
		
		

		this->arrayFaceGroup[this->faceGroupCount].endTexCoordsIndex = this->offsetTexCoords;			
		
		this->arrayFaceGroup[this->faceGroupCount].verticesCount = o->vertexcount;
		this->arrayFaceGroup[this->faceGroupCount].faceCount = o->facecount;
		this->arrayFaceGroup[this->faceGroupCount].texCoordsCount = o->texcoordscount;	
		
		this->faceGroupCount++;
	}
	
}
*/

void engine::FaceGroupAdd(object * o)
{
	int startVerticesIndex;
	int endVerticesIndex;
	float vert[3];
	unsigned int progid;

	DWORD * bufferVertices = this->bufferManager->GetBuffer(o->vertexcount*3);
	DWORD * bufferNormals = this->bufferManager->GetBuffer(o->vertexcount*3);
	DWORD * bufferUVMaps = NULL;
	if (o->texcoordscount > 0)
		bufferUVMaps = 	this->bufferManager->GetBuffer(o->texcoordscount*2);
	else memcpy(bufferUVMaps,o->texcoords,sizeof(float)*o->texcoordscount*2);
	
	if (o->textureCount > 0)
	{
		for (int i = 0;i < 	o->textureCount;i++)
		{
			if (o->textureid[i] == 0)
			{
				int ref = this->textureManager->add_texture(o->textureNames[i]);
				printf("New texture found: %s id=%d\n",o->textureNames[i],ref);
				o->textureid[i] = ref;
			}	
		}
	}
	
	if ((o->hasShader)&&(o->shader_prog_id == 0))
	{
		progid = this->shaderManager->add_shader(o->shader_prog);	
		printf("progid %d\n",progid);
		o->shader_prog_id = progid;
	}
	
	
	
	int offsetVertices = 0;
	startVerticesIndex = this->offsetVertices;
	
	matrix33f rotation;
	if ((o->hasShader == false)&&(o->shader_prog_id != 0))
	{
		rotation = matrix33f::get_rotationmatrix(o->get_orientation()[0],o->get_orientation()[1],o->get_orientation()[2]);	
	}

			
	for (int i = 0;i< o->vertexcount;i++)
	{
		vector3f translation;
		vector3f vertex;
		vector3f result;
		int size = 3;		
		
		if ((o->hasShader == false)&&(o->shader_prog_id != 0))
		{
			translation = vector3f(o->get_position());
			vertex = vector3f(o->vertices[i]);
			result = (rotation * vertex)+translation;
			result.copy_to(vert);		
				

			for (int j = 0;j < size;j++)
				bufferVertices[offsetVertices+j].f = vert[j];
		}
		else 
		{
				bufferVertices[offsetVertices].f = o->vertices[i][0];		
				bufferVertices[offsetVertices+1].f = o->vertices[i][1];	
				bufferVertices[offsetVertices+2].f = o->vertices[i][2];										
		}
			
		if (o->normals != NULL)
		{
			if ((o->hasShader == false)&&(o->shader_prog_id != 0))
			{			
				vertex = vector3f(o->normals[i]);
				result = (rotation * vertex);
				result.copy_to(vert);		
				
				
				for (int l = 0;l < size;l++)
					bufferNormals[offsetVertices+l].f = vert[l];	
			}
			else
			{
				bufferNormals[offsetVertices].f = o->vertices[i][0];		
				bufferNormals[offsetVertices+1].f = o->vertices[i][1];	
				bufferNormals[offsetVertices+2].f = o->vertices[i][2];
			}				
			
		}		
			
		offsetVertices += size;	
			
	}	
	
	
	endVerticesIndex = this->offsetVertices;		
	
	for(int i = 0;i < o->facegroupcount;i++)
	{
	
		this->arrayFaceGroup[this->faceGroupCount] = o->arrayfacegroup[i];
		
		for (int z = 0;z < o->textureCount;z++)
		{
			this->arrayFaceGroup[this->faceGroupCount].textureids[z] = o->textureid[z];

		}
		this->arrayFaceGroup[this->faceGroupCount].texturesCount = o->textureCount;			
		
		this->arrayFaceGroup[this->faceGroupCount].pNormalBuffer = bufferNormals;		
		
		this->arrayFaceGroup[this->faceGroupCount].pVertBuffer = bufferVertices;
		
		this->arrayFaceGroup[this->faceGroupCount].position[0] = o->get_position()[0];
		this->arrayFaceGroup[this->faceGroupCount].position[1] = o->get_position()[1];
		this->arrayFaceGroup[this->faceGroupCount].position[2] = o->get_position()[2];
		
		this->arrayFaceGroup[this->faceGroupCount].rotation[0] = o->get_orientation()[0];
		this->arrayFaceGroup[this->faceGroupCount].rotation[1] = o->get_orientation()[1];
		this->arrayFaceGroup[this->faceGroupCount].rotation[2] = o->get_orientation()[2];		
		/*
		for (int z = 0;z < o->texcoordscount;z++)
		{
			printf("u: %f v: %f\n",bufferUVMaps[z*2].f,bufferUVMaps[z*2+1].f);
		}
		*/
		if (o->shader_prog_id > 0)
		{
			this->arrayFaceGroup[this->faceGroupCount].shaderProgram = o->shader_prog_id;
			this->arrayFaceGroup[this->faceGroupCount].useShader = true;
		}
		this->arrayFaceGroup[this->faceGroupCount].uVertBufferOffset = offsetVertices;
	
	
		this->arrayFaceGroup[this->faceGroupCount].pUVMapBuffer = bufferUVMaps;
		this->arrayFaceGroup[this->faceGroupCount].uUVMapCount = o->texcoordscount;
		/*
		this->arrayFaceGroup[this->faceGroupCount].startFacesIndex = this->offsetFaceIndex;
		for (int j = o->arrayfacegroup[i].startFacesIndex;j < o->arrayfacegroup[i].endFacesIndex;j++)
		{	
			this->AddFaceIndexItem(3,o->arrayface[j].vertexindex);			
		}			
		this->arrayFaceGroup[this->faceGroupCount].endFacesIndex = this->offsetFaceIndex;
		*/
		
		this->arrayFaceGroup[this->faceGroupCount].uFaceIndexBufferOffset = 0;
		this->arrayFaceGroup[this->faceGroupCount].pFaceIndexBuffer = this->bufferManager->GetBuffer(o->arrayfacegroup[i].uFaceIndexBufferOffset*3);
		for (int j = 0;j < o->arrayfacegroup[i].uFaceIndexBufferOffset;j++)
		{	
			FaceGroupAddFaceIndexItem(&this->arrayFaceGroup[this->faceGroupCount],3,o->arrayface[j].vertexindex);			
		}			
		
		//printf("this->arrayFaceGroup[this->faceGroupCount].uFaceIndexBufferOffset=%d\n",this->arrayFaceGroup[this->faceGroupCount].uFaceIndexBufferOffset);
		//this->AddFaceIndexItem(3,facevertexindex);
		//facegrp->endFacesIndex = this->offsetFaceIndex;		
		
		//printf("%d\n",this->arrayFaceGroup[this->faceGroupCount].texturesCount);	
		if (o->hasShadow)
		{
			for (int b = 0;b < 10;b++)
			{
				if (this->arraylight[b] != 0)
					this->buildshadowvolume(this->arraylight[b],&this->arrayFaceGroup[this->faceGroupCount]);
			}
		}	
		this->faceGroupCount++;
	}

}

void engine::renderFaceGroups()
{
	for(int i = 0;i < this->faceGroupCount;i++)
	{
		this->calculate_lightings(&this->arrayFaceGroup[i]);
		
		this->FaceGroupRegisterBuffers(&this->arrayFaceGroup[i]);
		this->FaceGroupRender(&this->arrayFaceGroup[i]);
		this->FaceGroupUnregisterBuffers(&this->arrayFaceGroup[i]);
	}	
}


void engine::FaceGroupRender(FaceGroup * facegrp)
{
	//this->FaceGroupPrint(facegrp);
      EI_push_matrix();
	
	for (int i = 0;i < facegrp->texturesCount;i++)
	{
		EI_set_active_texture(i);
		EI_bind_2D_texture(facegrp->textureids[i]);
		//EI_bind_2D_texture(facegr);
		//this->bind_texture(facegrp->textureids[i]);
	 	//printf("kiki %d",facegrp->textureids[i]);
	}
	//printf("%d\n",facegrp->texturesCount);
	
		//EI_set_active_texture(0);
		//EI_bind_2D_texture(4);	

	//Rendu multi-passe de l'eclairage global
	if (facegrp->hasNormalMap)
	{
		
	}
	
	if (facegrp->useShader)
	{
		this->shaderManager->bind_shader(facegrp->shaderProgram);	
	}
	//else 
	{
    	//EI_translate(facegrp->position[0],facegrp->position[1],facegrp->position[2]);      
        EI_rotate(facegrp->rotation[0],facegrp->rotation[1],facegrp->rotation[2]); 		
        //EI_rotate(90.,0,0);
	}

	//printf("facegrp->texturesCount %d\n",facegrp->texturesCount);
	//Rendu final
	//facegrp->texCoordsLayerCount
	
	
	if (facegrp->verticesPerFace == 3)
	EI_render_triangle(facegrp->elementBufferName,facegrp->vertexBufferName,facegrp->texcoordsBufferName,
		facegrp->colorBufferName,facegrp->faceCount,facegrp->texturesCount);	
		/*
	else
	EI_render_quad(facegrp->elementBufferName,facegrp->vertexBufferName,facegrp->texcoordsBufferName,
		facegrp->colorBufferName,facegrp->faceCount,facegrp->texCoordsLayerCount);
	*/
	if (facegrp->useShader)
	{
		this->shaderManager->bind_shader(0);	
	}	
	
	
	for (int i = 0;i < facegrp->texturesCount;i++)
	{
		EI_set_inactive_texture(i);
	}
		
	
      EI_pop_matrix();	
}

/*
void engine::ShadowFaceGroupInit(FaceGroup * facegrp)
{
	facegrp->hasTexture = false;
	facegrp->hasNormalMap = false;
	//facegrp->startFacesIndex = this->offsetFaceIndex;
	//facegrp->endFacesIndex = this->offsetFaceIndex;
	facegrp->faceCount = 0;
	facegrp->texturesCount = 0;
	facegrp->useShader = 0;
	//facegrp->startTexCoordsIndex =0;
	//facegrp->endTexCoordsIndex =0;	
	facegrp->texCoordsCount = 0;
	facegrp->verticesPerFace = 4;
	//facegrp->startVerticesIndex = this->offsetVertices;
	//facegrp->endVerticesIndex = this->offsetVertices;
	facegrp->verticesCount = 0;
	facegrp->texCoordsLayerCount = 0;
	facegrp->elementBufferName = 0;
	facegrp->vertexBufferName = 0;
	facegrp->texcoordsBufferName = 0;
	facegrp->colorBufferName = 0;	
}
*/
DWORD s_buffer1[100000];
DWORD s_buffer2[100000];


void engine::ShadowTriangleFaceGroupInit(FaceGroup * facegrp)
{
	facegrp->hasTexture = false;
	facegrp->hasNormalMap = false;
	//facegrp->startFacesIndex = this->offsetFaceIndex;
	//facegrp->endFacesIndex = this->offsetFaceIndex;
	facegrp->faceCount = 0;
	facegrp->texturesCount = 0;
	facegrp->useShader = 0;
	//facegrp->startTexCoordsIndex =0;
	//facegrp->endTexCoordsIndex =0;	
	facegrp->texCoordsCount = 0;
	facegrp->verticesPerFace = 3;
	//facegrp->startVerticesIndex = this->offsetVertices;
	//facegrp->endVerticesIndex = this->offsetVertices;
	facegrp->verticesCount = 0;
	facegrp->texCoordsLayerCount = 0;
	facegrp->elementBufferName = 0;
	facegrp->vertexBufferName = 0;
	facegrp->texcoordsBufferName = 0;
	facegrp->colorBufferName = 0;	
	//facegrp->pFaceIndexBuffer = this->bufferManager->GetBuffer(10000);
	//facegrp->pVertBuffer = this->bufferManager->GetBuffer(10000);	
	facegrp->pFaceIndexBuffer = s_buffer1;
	facegrp->pVertBuffer = s_buffer2;		
	facegrp->pColorBuffer = NULL;
	facegrp->pUVMapBuffer = NULL;
}


void engine::FaceGroupRegisterBuffers(FaceGroup * facegrp)
{
	unsigned int * pFacesIndex;
	float * pVertices;	
	float * pColors;
	float * pTexCoords;
	if ((facegrp->faceCount > 0)&&(facegrp->verticesCount > 0))
	{

		//pFacesIndex = this->bufferFaceIndex + sizeof(int)*facegrp->faceCount;
		pFacesIndex = (unsigned int*)facegrp->pFaceIndexBuffer;
		EI_RegisterFaceBuffer(&facegrp->elementBufferName,sizeof(int)*facegrp->faceCount*facegrp->verticesPerFace,pFacesIndex);
		//printf("Element Buffer Name %d\n",facegrp->elementBufferName);
		//pVertices = this->bufferVertices + sizeof(float)*facegrp->verticesCount;	
		pVertices = (float*)facegrp->pVertBuffer;	
		EI_RegisterVertexBuffer(&facegrp->vertexBufferName,facegrp->verticesCount*3*sizeof(float),pVertices);		
		
		if (facegrp->pColorBuffer != NULL)
		{
			pColors = (float*)facegrp->pColorBuffer;	
			EI_RegisterColorBuffer(&facegrp->colorBufferName,facegrp->verticesCount*3*sizeof(float),pColors);	
		}
		
		if (facegrp->pUVMapBuffer != NULL)
		{
			pTexCoords = (float*)facegrp->pUVMapBuffer;	
			EI_RegisterTexCoordsBuffer(&facegrp->texcoordsBufferName,facegrp->verticesCount*2*sizeof(float),pTexCoords);
		}					
	}
}

void engine::FaceGroupUnregisterBuffers(FaceGroup * facegrp)
{
	if (facegrp->elementBufferName != 0)
	{
		EI_UnregisterFaceBuffer(facegrp->elementBufferName);
	}
	
	if (facegrp->vertexBufferName != 0)
	{
		EI_UnregisterVertexBuffer(facegrp->vertexBufferName);
	}
	
	
	if (facegrp->colorBufferName != 0)
	{
		EI_UnregisterColorBuffer(facegrp->colorBufferName);
	}	
	
	if (facegrp->texcoordsBufferName != 0)
	{
		EI_UnregisterTexCoordsBuffer(facegrp->texcoordsBufferName);
	}			
}

/*
void engine::FaceGroupAddTriangle(FaceGroup * facegrp,float * a,float * b,float *c)
{
	
	int facevertexindex[3];	
	for(int i = 0;i < 3;i++)
		facevertexindex[i] = this->offsetVertices+i;
	
	this->AddVertexItem(3,a,facegrp->pVertBuffer);
	this->AddVertexItem(3,b,facegrp->pVertBuffer);
	this->AddVertexItem(3,c,facegrp->pVertBuffer);
	//facegrp->endVerticesIndex = this->offsetVertices;


	this->AddFaceIndexItem(3,facevertexindex,facegrp->pFaceIndexBuffer);
	facegrp->verticesCount += 3;
	
	//facegrp->endFacesIndex = this->offsetFaceIndex;		
	facegrp->faceCount++;
}
*/

/*
void engine::FaceGroupAddQuad(FaceGroup * facegrp,float * a,float * b,float *c,float * d)
{
	//this->arrayFaceGroup[this->faceGroupCount].startVerticesIndex = this->offsetVertices;
	int facevertexindex[4];	
	//printf("->b %d\n",this->offsetFaceIndex);
	for(int i = 0;i < 4;i++)
		facevertexindex[i] = this->offsetVertices+i*3;
	
	//this->bufferFaceIndex[this->offsetFaceIndex] = this->offsetVertices;
	

	this->AddVertexItem(3,a,facegrp->pVertBuffer);
	this->AddVertexItem(3,b,facegrp->pVertBuffer);
	this->AddVertexItem(3,c,facegrp->pVertBuffer);
	this->AddVertexItem(3,d,facegrp->pVertBuffer);
	
	//facegrp->endVerticesIndex = this->offsetVertices;

	this->AddFaceIndexItem(4,facevertexindex);
	facegrp->verticesCount += 4;
	
	//facegrp->endFacesIndex = this->offsetFaceIndex;		
	//printf("facegrp->endFacesIndex %d\n",facegrp->endFacesIndex);
	facegrp->faceCount++;
	//this->offsetFaceIndex++;
}
*/

void engine::FaceGroupPrint(FaceGroup * facegrp)
{
	printf("-------------FaceGroup-------------\n");
	printf("facegrp->hasTexture %d\n",facegrp->hasTexture);
	printf("facegrp->hasNormalMap %d\n",facegrp->hasNormalMap);
	//printf("facegrp->startFacesIndex %d\n",facegrp->startFacesIndex);
	//printf("facegrp->endFacesIndex %d\n",facegrp->endFacesIndex); 
	printf("facegrp->faceCount %d\n",facegrp->faceCount);
	printf("facegrp->texturesCount %d\n",facegrp->texturesCount);
	printf("facegrp->useShader %d\n",facegrp->useShader);
	//printf("facegrp->startTexCoordsIndex %d\n",facegrp->startTexCoordsIndex);
	//printf("facegrp->endTexCoordsIndex %d\n",facegrp->endTexCoordsIndex);	
	printf("facegrp->texCoordsCount %d\n",facegrp->texCoordsCount);
	printf("facegrp->verticesPerFace %d\n",facegrp->verticesPerFace);
	//printf("facegrp->startVerticesIndex %d\n",facegrp->startVerticesIndex);
	//printf("facegrp->endVerticesIndex %d\n",facegrp->endVerticesIndex);
	printf("facegrp->verticesCount %d\n",facegrp->verticesCount);
	printf("facegrp->texCoordsLayerCount %d\n",facegrp->texCoordsLayerCount);
	printf("facegrp->elementBufferName %d\n",facegrp->elementBufferName);
	printf("facegrp->vertexBufferName %d\n",facegrp->vertexBufferName);
	printf("facegrp->texcoordsBufferName %d\n",facegrp->texcoordsBufferName);
	printf("facegrp->colorBufferName %d\n",facegrp->colorBufferName);		
}

void engine::BuffersReset(void)
{
	this->offsetVertices = 0;

//	this->offsetFaceIndex = 0;
//	this->offsetTexCoords = 0;
}

int engine::getFaceCount()
{
	return this->facecount;	
}

int engine::getVertexCount()
{
	return this->vertexcount;	
}
int engine::getMobileCount()
{
	return this->mobiles->get_count();	
}

int engine::getObjectCount()
{
	return this->objects->get_count();	
}
/*
int engine::getQuadCount()
{
	//return this->quadfacecount;	
}
*/
/*
int engine::getTriangleCount()
{
	return this->trianglefacecount;	
}
*/


void engine::translateCamera(float x,float y,float z)
{
	this->cam->translate_x(x);
	this->cam->translate_y(y);
	this->cam->translate_z(z);		
}
	
void engine::rotateCamera(float ax,float ay, float az)
{
	this->cam->rotate_x(ax);
	this->cam->rotate_y(ay);
	this->cam->rotate_z(az);		
}
