#include "matrix33f.h"
#include "math.h"

static float cosd[360];
static float sind[360];

int init(void)
{
	for (int i = 0;i < 361;i++)
	{
		cosd[i] = cos((float)i);
		sind[i] = sin((float)i);
		
	}
	return 0;
}


matrix33f::matrix33f(const float mat[9]) {
	for (int i = 0;i < 9;i++)
		t[i] = mat[i];
}

matrix33f::matrix33f() {
}

matrix33f::~matrix33f() {
}

float & matrix33f::operator() (int i,int j) {
	return t[i+j*3];
}

float & matrix33f::operator() (int i) {
	return t[i];
}

matrix33f matrix33f::operator * (matrix33f m) {
	matrix33f mres;
	matrix33f::mul(mres, *this, m);
	return mres;
}

void matrix33f::add(matrix33f & result, matrix33f m1, matrix33f m2) {
	for (int i = 0;i < 9;i++)
		result(i) = m1(i) + m2(i);
}

void matrix33f::sub(matrix33f & result, matrix33f m1, matrix33f m2) {
	for (int i = 0;i < 9;i++)
		result(i) = m1(i) - m2(i);
}

void matrix33f::mul(matrix33f & result, matrix33f m1, matrix33f m2) {
	for (int i = 0;i < 3;i++)
		for (int j = 0;j < 3;j++) {
			result(i, j) = m1(i, 0) * m2(0, j) + m1(i, 1) * m2(1, j) + m1(i, 2) * m2(2, j);
		}
}

matrix33f matrix33f::get_identitymatrix() {
	float f[9] = {1., 0., 0., 0., 1., 0., 0., 0., 1.};
	return matrix33f(f);
}

matrix33f matrix33f::get_rotationmatrix(float ax, float ay, float az) {
	float cosax = cos(ax*3.1415/180.);
	float sinax = sin(ax*3.1415/180.);
	float cosay = cos(ay*3.1415/180.);
	float sinay = sin(ay*3.1415/180.);
	float cosaz = cos(az*3.1415/180.);
	float sinaz = sin(az*3.1415/180.);
	const float tx[9] = {1, 0, 0, 0, cosax, -sinax, 0, sinax, cosax};
	const float ty[9] = {cosay, 0, sinay, 0, 1., 0., -sinay, 0, cosay};
	const float tz[9] = {cosaz, -sinaz, 0, sinaz, cosaz, 0, 0, 0, 1};
	matrix33f Rx = matrix33f(tx);
	matrix33f Ry = matrix33f(ty);
	matrix33f Rz = matrix33f(tz);
	return Rx * Ry * Rz;

}

vector3f matrix33f::operator * (vector3f v) {
	vector3f vres;
	vres(0) = (*this)(0, 0) * v(0) + (*this)(1, 0) * v(1) + (*this)(2, 0) * v(2);
	vres(1) = (*this)(0, 1) * v(0) + (*this)(1, 1) * v(1) + (*this)(2, 1) * v(2);
	vres(2) = (*this)(0, 2) * v(0) + (*this)(1, 2) * v(1) + (*this)(2, 2) * v(2);
	return vres;
}
