#include "vector3f.h"
#include "math.h"

vector3f::~vector3f()
{
	
}

vector3f::vector3f()
{
	this->v[0] = 0.;
	this->v[1] = 0.;
	this->v[2] = 0.;	
}

vector3f::vector3f(float v[3])
{
	this->v[0] = v[0];
	this->v[1] = v[1];
	this->v[2] = v[2];;		
}

vector3f::vector3f(float x,float y,float z)
{
	this->v[0] = x;
	this->v[1] = y;
	this->v[2] = z;	
}

inline float& vector3f::operator()(unsigned int i)
{
    return this->v[i];
}

float * vector3f::get(void)
{
	return this->v;
}

 void vector3f::add(vector3f & result,vector3f v1,vector3f v2)
{
  result(0) = v1(0) + v2(0);
  result(1) = v1(1) + v2(1);	
  result(2) = v1(2) + v2(2);	
}

void vector3f::add(vector3f & result,vector3f v1,vector3f v2,float range)
{
  result(0) = v1(0) + v2(0);
  result(1) = v1(1) + v2(1);	
  result(2) = v1(2) + v2(2);	
	if (result(0) < 0.)
	{
		result(0)  = 0.;
	}
	if (result(1) < 0.)
	{
		result(1)  = 0.;
	}
	if (result(2) < 0.)
	{
		result(2)  = 0.;
	}	
	if (result(0) > range)
	{
		result(0)  = range;
	}
	if (result(1) > range)
	{
		result(1)  = range;
	}
	if (result(2) < range)
	{
		result(2)  = range;
	}		
}

void vector3f::sub(vector3f & result,vector3f v1,vector3f v2)
{
  result(0) = v1(0) - v2(0);
  result(1) = v1(1) - v2(1);	
  result(2) = v1(2) - v2(2);	
}

inline float vector3f::mul(vector3f v1,vector3f v2)
{
  return v1(0) * v2(0) + v1(1) * v2(1) + v1(2) * v2(2);	
}

inline void vector3f::mul(vector3f & result,vector3f v1,float a)
{
  result(0) = v1(0) * a;
  result(1) = v1(1) * a;	
  result(2) = v1(2) * a;		
}

void vector3f::vect(vector3f & result,vector3f v1,vector3f v2)
{
	result(0) = v1(1) * v2(2) - v1(2) * v2(1);
	result(1) = v1(2) * v2(0) - v1(0) * v2(2);
	result(2) = v1(0) * v2(1) - v1(1) * v2(0); 
}


/*
inline vector3f&  vector3f::operator =(vector3f v)
{

}
*/
bool vector3f::is_null(void)
{
	if ( this->v[0] == 0.)
		if ( this->v[1] == 0.)
			if ( this->v[2] == 0.)
				return true;
	return false;
}


inline void vector3f::copy(vector3f & result,vector3f v)
{
  result(0) = v(0);
  result(1) = v(1);	
  result(2) = v(2);		
}

float vector3f::distance(vector3f v1,vector3f v2)
{
	return sqrt( ((v1(0)-v2(0))*(v1(0)-v2(0)))+((v1(1)-v2(1))*(v1(1)-v2(1)))+((v1(2)-v2(2))*(v1(2)-v2(2)) ) );
}

void vector3f::normalize(vector3f & result,vector3f v)
{
	vector3f nullvect = vector3f(0.,0.,0.);
	if ( v.is_null() )
	{
		result(0) = 0.;
		result(1) = 0.;	
		result(2) = 0.;		
		return;
	}
	float d = distance(v,nullvect);
	result(0) = v(0)/d;
	result(1) = v(1)/d;	
	result(2) = v(2)/d;
}

void vector3f::normalize(void)
{
  vector3f::normalize(*this,*this);
}
  
float vector3f::length(void)
{
  return vector3f::distance(*this,vector3f(0.,0.,0.));
}

float vector3f::operator *(vector3f v)
{
	return vector3f::mul(*this,v);
}

void vector3f::print(void)
{
	printf("Vector %f %f %f\n",this->v[0],this->v[1],this->v[2]);
}

vector3f vector3f::normal(float * p0,float * p1,float * p2)
{
	vector3f res;
	vector3f vm1;
	vector3f vm2;
	vector3f v0 = vector3f(p0);
	vector3f v1 = vector3f(p1);
	vector3f v2 = vector3f(p2);       
	vector3f::sub(vm1,v1,v0);
	vector3f::sub(vm2,v2,v0);
	vector3f::vect(res,vm1,vm2);
	res.normalize();
	return res;
	      	
}

vector3f vector3f::operator *(float a)
{
	vector3f res;
	res(0) = this->v[0]  * a;
	res(1) = this->v[1]  * a;
	res(2) = this->v[2]  * a;	
	return res;
}

vector3f vector3f::operator -(vector3f v)
{
	vector3f res;
	res(0) = this->v[0]  - v(0);
	res(1) = this->v[1]  - v(1);
	res(2) = this->v[2]  - v(2);	
	return res;
}

vector3f vector3f::operator +(vector3f v)
{
	vector3f res;
	res(0) = this->v[0] + v(0);
	res(1) = this->v[1] + v(1);
	res(2) = this->v[2] + v(2);	
	return res;
}

void vector3f::copy_to(float * v)
{
	v[0] = this->v[0];
	v[1] = this->v[1];
	v[2] = this->v[2];	
}
